import java.util.*;
import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

public abstract class AbstractDataField extends DataNode implements DataField {

/*******
* data *
*******/
protected String name;
protected Number value;
protected String comment;

protected Number max_value;
protected Number min_value;

protected int size;

/**********************************************************************
* constructor 
**********************************************************************/
AbstractDataField(String name, int size, String comment) {

    setAllowsChildren(false);

    this.name    = name;
    this.size    = size;
    this.comment = comment;

    value = new Integer(0);

} // end of constructor

/**********************************************************************
* accessors 
**********************************************************************/
public String name(           ) {      return name; }
public void   name(String name) { this.name = name; }

public Number value(            ) {       return value; }
public void   value(Number value) { 
    if( isInValidRange(value) )  this.value = validateType(value);
}

public String comment(              ) {         return comment; }
public void   comment(String comment) { this.comment = comment; }

public String formattedValue() { 
    if(value() == null) return null;
    return value().toString(); 
}

public int size() { return size; }
public void size(int size) { this.size = size; System.out.println("Setting size in "+this+" to"+size);}


public Number minValue() { return min_value; }
public Number maxValue() { return max_value; }

public void minValue(Number min) { min_value = min; }
public void maxValue(Number max) { max_value = max; }


/**********************************************************************
* check if a given Number is a valid value for this field 
**********************************************************************/
public abstract boolean isInValidRange(Number n);





/**********************************************************************
* return a Number of the appropriate type for this field
* returns null if the data type of this field is unknown or weird.
**********************************************************************/
public abstract Number validateType(Number n);

/**********************************************************************
* handy string representation 
**********************************************************************/
public String toString() {

return name+" = "+value+" / "+comment; 

}

/**********************************************************************
* component for rendering 
**********************************************************************/
public Component displayComponent(boolean selected, boolean expanded, 
                                  int row, boolean hasFocus) {

    return new DataFieldRenderer(this);

} // end of displayComponent method




/**********************************************************************
* method to read this data field 
**********************************************************************/
public abstract void read(BitInputStream in) throws IOException;



/**********************************************************************
* method to write this data field 
**********************************************************************/
public abstract void write(BitOutputStream out) throws IOException;


/***************************************************************************
* editing methods 
***************************************************************************/
public DataNodeEditor getEditor(int row) {

    return new DataFieldRenderer(this,true);

}

/***************************************************************************
* allow a new value to be set  - called by tree after editing
***************************************************************************/
public void setUserObject(Object o) { 

try {
    /********************************************************************
    * interpret the result as a Map containing key - string value pairs *
    ********************************************************************/
    Map result = (Map)o;

    Number value = interpretStringAsNumber((String)result.get("value") );
    if(value != null) value(value);

    String name = (String)result.get("name");
    if(name != null) name(name);

    String comment = (String)result.get("comment");
    if(comment != null) comment(comment);

System.out.println("parsing size");
    Number size = interpretStringAsNumber((String)result.get("size") );
    if(size != null ) size(size.intValue() );

} catch(ClassCastException e) {}


} // end of setUserObject method

/***************************************************************************
* utility method for converting a string to the best number representation *
***************************************************************************/
private static Number interpretStringAsNumber(String string ) {

    if(string==null) return null;

System.out.println("string=|"+string+"|");

    try { return new Integer(string); }
    catch (NumberFormatException e) {}
    
    try {return new Long(string); }
    catch (NumberFormatException e) {}
    
    
    try {return new Double(string); }
    catch (NumberFormatException e) {}
    
    return null;

} // end of interpretStringAsNumber

/***************************************************************************
* Static method which creates an appropriate type data field based on its
* ITOS type. The resulting class will have its name and comment fields unset
***************************************************************************/
public static AbstractDataField createByITOSType(String type, int size) {

/*****************
* signed integer *
*****************/
if(type.equals("SB") ||
   type.startsWith("SI") ||
   type.startsWith("SLI") ||
   type.startsWith("I") ) {

    if(type.endsWith("085") || type.endsWith("21") ) {
        /****************
        * little endian *
        ****************/
        return new ReverseSignedIntField(null, size, null);
    } else {
        /*************
        * big endian *
        *************/
        return new SignedIntField(null, size, null);
    }
}

/*******************
* unsigned integer *
*******************/
if(type.equals("UB") ||
   type.startsWith("UI") ||
   type.startsWith("ULI") ||
   type.startsWith("U") ) {

    if(type.endsWith("085") || type.endsWith("21") ) {
        /****************
        * little endian *
        ****************/
        return new ReverseUnsignedIntField(null, size, null);
    } else {
        /*************
        * big endian *
        *************/
        return new UnsignedIntField(null, size, null);
    }


}

/********
* float *
********/
if(type.startsWith("SFP") || type.equals("F1234") ) {

    return new FloatField(null, null);

}

/**********************
* little endian float *
**********************/
if(type.equals("F4321") ) {

    return new ReverseFloatField(null, null);

}


/*********
* double *
*********/
if(type.startsWith("DFP") || type.equals("F12345678") ) {
    return new DoubleField(null, null);

}

/*********
* string *
*********/
if(type.startsWith("S1") || type.equals("CHAR") || type.equals("S") ) {
    return new ASCIIDataField(null, size, null);
}


/***************
* unknown type *
***************/
return null;


} // end of createByITOSType method

/****************************************************************************
* substitute the value with a byte-reversed value of itself
* This method is used for little endian fields
****************************************************************************/
protected void byteReverseValue() {

int nbytes = (size()+7)/8;

if(nbytes>4) {
    /***************
    * long storage *
    ***************/
    long big_end = value.longValue();
    long reverse=0;
    for(int i=0; i<nbytes; ++i) {
        reverse <<=8;
        reverse |= (big_end & 0xff);
        big_end >>=8;
    }

    value = new Long(reverse);

} else if(nbytes>1) {
    /***************
    * int storage *
    **************/
    int big_end = value.intValue();
    int reverse=0;
    for(int i=0; i<nbytes; ++i) {
        reverse <<=8;
        reverse |= (big_end & 0xff);
        big_end >>=8;
    }

    value = new Integer(reverse);

}  

} // end of byteReverseValue method


/***************************************************************************
* main method for testing 
***************************************************************************/
public static void main(String[] args) throws Exception {
}

} // end of AbstractDataField class


