/***************************************************************************
* This is a class for writing individual bits to a data stream.
* Most significant bits come first in the stream
***************************************************************************/
import java.io.*;

public class BitOutputStream extends DataOutputStream {

int buffer;
int nbits;

/***************************************************************************
*
***************************************************************************/
public BitOutputStream(OutputStream out) {

super(out);

buffer=0;
nbits=0;

} // end of constructor

/***************************************************************************
* write the least significant bit of a given integer to the stream
***************************************************************************/
public void writeBit(int bit) throws IOException {

/******************************
* put the bit into the buffer *
******************************/
buffer <<= 1;
buffer |= (bit & 1);
++nbits;

/****************************************
* check if we need to empty the buffer *
****************************************/
if(nbits == 8) {
    write(buffer);
    buffer=0;
    nbits=0;
}


//System.out.println("wrote: "+bit+" buffer="+buffer+" nbits="+nbits);

} // end of writeBit

/***************************************************************************
* write a 1 to the stream if the boolean is true
* otherwise write a 0.
***************************************************************************/
public void writeBit(boolean bit) throws IOException {

if(bit) writeBit(1);
else    writeBit(0);

}


/***************************************************************************
* write the n least significant bits of an integer to the stream
***************************************************************************/
public void writeBits(int bits, int n) throws IOException {

for(int i=0; i<n; ++i) {

    int shift = n-1;
    writeBit(bits>>shift);
    bits <<= 1;

}

} // end of writeBits method

/***************************************************************************
* write the n least significant bits of an integer to the stream
***************************************************************************/
public void writeBits(long bits, int n) throws IOException {

for(int i=0; i<n; ++i) {

    int shift = n-1;
    writeBit((int)(bits>>shift));
    bits <<= 1;

}

} // end of writeBits method


/****************************************************************************
* flush any remaining bits in the buffer.
* Note that if the buffer is not full, then the unset bits 
****************************************************************************/
public void close() throws IOException{

write(buffer);
super.close();

} // end of close method


/****************************************************************************
* returns true if the bit buffer is empty - i.e. if the next read would
* come from the most significant bit of a byte.
***************************************************************************/
public boolean isByteAligned() { return nbits==0; }



} // end of BitInputStream class
