import java.io.*;
import java.lang.reflect.*;

/*****************************************************************************
* This is a filename filter which passes all ".class" bytecode files
* containing the code for a given base class and all its sub-classes.
* Only inner classes and abstract classes do not pass.
*****************************************************************************/
public class ClassFilenameFilter extends FilenameSuffixFilter {

/*******
* data *
*******/
Class base_class;

/***************************************************************************
* constructor
***************************************************************************/
ClassFilenameFilter(Class base_class ) {

super(".class");
this.base_class = base_class;

} // end of constructor

/***************************************************************************
* this method implements the FilenameFilter method
***************************************************************************/
public boolean accept(File dir, String name) {

if(! super.accept(dir,name) ) return false;
if(name.indexOf('$') != -1 )  return false;

try {
    Class c = Class.forName(name.substring(0,name.length()-6 ) );

    return base_class.isAssignableFrom(c) && 
           !Modifier.isAbstract(c.getModifiers());

} catch(ClassNotFoundException e) { return false; }


} // end of accept method


} // end of ClassFilenameFilter class
