import java.io.*;
import java.util.*;
import javax.swing.tree.*;
import java.awt.*;
import javax.swing.*;


public class DataBlock extends DataNode {

/*************
* class data *
*************/
static FileSearchPath path = FileSearchPath.getClassPath();

static void setFileSearchPath(FileSearchPath new_path) { path=new_path; }
static FileSearchPath getFileSearchPath() { return path; }

/*******
* data *
*******/
private String name;
private Map  map;



/***************************************************************************
* constructor - empty data block 
***************************************************************************/
DataBlock() {

super();
setAllowsChildren(true);

name="";
map = new HashMap();

} // end of empty constructor

/***************************************************************************
* constructor - from an ITOS-style database file 
***************************************************************************/
DataBlock(String name) throws IOException {

this();
this.name = name;
setStructureIsMutable(false);

setUp(name+".db");

} // end of constructor from a database file

/**********************************************************************
* accessors 
**********************************************************************/
public String name(           ) {      return name; }
public void   name(String name) { this.name = name; }


/**********************************************************************
* string representation 
**********************************************************************/
public String toString() { return name(); }



/***************************************************************************
* add a field at the end of this DataBlock
***************************************************************************/
public void add(DataNode node) {

    /****************************
    * add to the tree structure *
    ****************************/
    insert(node, getChildCount() );

    /********************************************************
    * add to the name-keyed hash map if this is a DataField *
    ********************************************************/
    try {
        DataField field = (DataField)node;
        map.put(field.name(), field);
    } catch(ClassCastException e) {}
}

/***************************************************************************
* return a field either by position or by value
***************************************************************************/
public DataNode get(int index) { return (DataNode)getChildAt(index); }
public DataField get(String name) { return (DataField)map.get(name); }




/***************************************************************************
* parse an ITOS-style database file 
***************************************************************************/
private void setUp(String filename) throws IOException {

String delimiter = ",";

/****************
* open the file *
****************/
File file = path.find(filename);
if(file==null) return;

BufferedReader in = new BufferedReader(new FileReader(file));

String line;
while((line= in.readLine()) != null) {

    line = line.trim();

    /****************
    * skip comments *
    ****************/
    if(line.startsWith("#") ) continue;

    /*******************
    * skip blank lines *
    *******************/
    if(line.length() == 0) continue;

    /***************************************
    * parse the first field and interpret  
    * the rest of the line accordingly
    ***************************************/
    StringTokenizer tokenizer = new NullTokenizer(line,delimiter);
    String first = tokenizer.nextToken();

    if(first.compareToIgnoreCase("DEL")==0) {
        /***********************
        * change the delimiter *
        ***********************/
        delimiter=tokenizer.nextToken();

    } else if(first.compareToIgnoreCase("PKT")==0) {
        /*************************
        * read a "packet" record *
        *************************/
        tokenizer.nextToken();
        String name = tokenizer.nextToken().trim();
        tokenizer.nextToken();
        tokenizer.nextToken();
        tokenizer.nextToken();
        String type = tokenizer.nextToken().trim();

        /**********************************************
        * we don't really need the start bit and byte *
        **********************************************/
        int start_byte=-1;
        int start_bit=-1;
        try { start_byte = Integer.parseInt(tokenizer.nextToken().trim()); }
        catch(NumberFormatException e) {}

        try { start_bit  = Integer.parseInt(tokenizer.nextToken().trim());
        } catch(NumberFormatException e) {}

        /*********************************************************
        * throw an IO exception if we can't parse the size field *
        *********************************************************/
        int size;
        try { size = Integer.parseInt(tokenizer.nextToken().trim()); }
        catch(NumberFormatException e) { 
            throw new IOException("Couldn't parse size field of \""+name+"\"");
        }

        tokenizer.nextToken();
        tokenizer.nextToken();
        String comment = tokenizer.nextToken().trim();

        if((comment.startsWith("\"") && comment.endsWith("\"")) ||
           (comment.startsWith("'" ) && comment.endsWith("'") )   ) {
            /*****************************
            * remove quotes from comment *
            *****************************/
            comment = comment.substring(1,comment.length()-1);
        }

      //  System.out.println("packet record name="+name+" type="+type+" "+comment);

        /************************************
        * 
        ************************************/
        DataNode node;
        if(type.startsWith("c")) {
            /*************************************
            * this entry is a hierarchical class *
            *************************************/
            node = createByName(type.substring(1));
            if(node == null ) throw new IOException("Invalid type "+type);
        } else {
            /****************************
            * this is an ITOS data type *
            ****************************/
            node = AbstractDataField.createByITOSType(type, size);
            node.setStructureIsMutable(false);
            if(node==null) throw new IOException("Invalid type "+type);
        }
            
        /****************************************************
        * if this is a data field, set the name and comment *
        ****************************************************/
        try {
            DataField field = (DataField)node;
            field.name(name);
            field.comment(comment);

        } catch(ClassCastException e) {}

        /****************************
        * add the node to the block *
        ****************************/
        add(node);

    } // end if this is a PKT record

} // end of loop over lines

/************************************************************************
* explicitly close the file to avoid having too many files open at once *
************************************************************************/
in.close();


} // end of setUp method



/***************************************************************************
* read the datablock 
***************************************************************************/
public void read(BitInputStream in) throws IOException {

for(Enumeration enum = children(); enum.hasMoreElements(); ) {
    DataNode node = (DataNode)enum.nextElement();

    node.read(in);
}


} // end of read method

/***************************************************************************
* read the datablock 
***************************************************************************/
public void write(BitOutputStream out) throws IOException {

for(Enumeration enum = children(); enum.hasMoreElements(); ) {
    DataNode node = (DataNode)enum.nextElement();

    node.write(out);
}


} // end of read method

/***************************************************************************
* return the size of the datablock in bits 
***************************************************************************/
public int size() {

int size=0;

for(Enumeration enum = children(); enum.hasMoreElements(); ) {
    DataNode node = (DataNode)enum.nextElement();

    size += node.size();
}

return size;


} // end of size method




/***************************************************************************
* main method for testing 
***************************************************************************/
public static void main(String[] args) throws Exception {

DataBlock block = new DataBlock("SwiftAPID1158");

System.exit(0);

DataBlock head1 = new DataBlock("CCSDSHead.db");
DataBlock head2 = new DataBlock("SwiftHead2.db");

BitInputStream in = new BitInputStream(new FileInputStream("zwift_0_0_0.ldp"));
head1.read(in);
head2.read(in);

for(int i=0; i<8;++i) {
    System.out.println(head1.get(i));
}


JFrame frame = new JFrame();
JTree tree = new JTree(head2);
JScrollPane scrollPane = new JScrollPane(tree);
frame.getContentPane().add(scrollPane, BorderLayout.CENTER);

frame.pack();
frame.setVisible(true);



} // end of main method


} // end of DataBlock class
