import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;

public class DataFieldRenderer extends JPanel implements DataNodeEditor, 
                                                         ActionListener {
boolean edit;

JTextField name_input;
JTextField value_input;
JTextField comment_input;
JTextField size_input;

java.util.List listeners;


/***************************************************************************
* constructor
***************************************************************************/
DataFieldRenderer(DataField field) { this(field,false); }
DataFieldRenderer(DataField field, boolean edit) {

listeners = new ArrayList();

this.edit=edit;

setBackground(Color.white);
setLayout(new GlobalLayout() );

String name_string = field.name();
if(name_string==null) name_string="????";

String comment_string = field.comment();
if(comment_string==null) comment_string = "";

JLabel equals  = new JLabel(" = ");
JLabel slash   = new JLabel(" / ");


if(edit) {
    /**********************
    * set up as an editor *
    **********************/
    if(field.structureIsMutable() ) {
        name_input = new JTextField(name_string);
        name_input.addActionListener(this);
        add(name_input);

    } else {
        add(new JLabel(name_string) );
    }

    add(equals);

    value_input = new JTextField(field.formattedValue() );
    value_input.addActionListener(this);
    value_input.setHorizontalAlignment(JTextField.RIGHT);
    add(value_input);



    add(slash);

    if(field.structureIsMutable() ) {
        comment_input = new JTextField(comment_string);
        comment_input.addActionListener(this);
        add(comment_input);
    } else {
        add(new JLabel(comment_string));
    }

    /**************
    * size editor *
    **************/
    if(field.structureIsMutable() ) {
        add(new JLabel(" (") );

        size_input = new JTextField(""+field.size() );
        size_input.addActionListener(this);
        size_input.setHorizontalAlignment(JTextField.RIGHT);
        add(size_input);

        add(new JLabel(" bits)") );
    }

} else {
    /*************************
    * set up as display only *
    *************************/
    add( new JLabel(name_string) );
    add(equals);

    JLabel value = new JLabel(field.formattedValue());
    value.setHorizontalAlignment(SwingConstants.RIGHT);

    add(value);

    
    add(slash);
    add( new JLabel(comment_string) );

    ((JLabel)value).setHorizontalAlignment(SwingConstants.RIGHT);   

}


/**************************************
* set the size of the whole container *
**************************************/
//setPreferredSize(new Dimension(comment_start+comment_width, height));

} // end of constructor

/***************************************************************************
* the following method implements ActionListener
* we listen to the text field input boxes and pass events on to 
* anyone listening to this object
***************************************************************************/
public void actionPerformed(ActionEvent e) {

java.util.List copy = new ArrayList(listeners);
for(Iterator it=copy.iterator(); it.hasNext(); ) {
    ActionListener l = (ActionListener)it.next();

    l.actionPerformed( new ActionEvent(this,0,"Done editing") );

}



} // end of actionPerformed method



/***************************************************************************
* the following methods implement DataNodeEditor 
***************************************************************************/
public void    addActionListener(ActionListener l) { listeners.add(l);    }
public void removeActionListener(ActionListener l) { listeners.remove(l); }

/**************************************************************************
* returns the current state of the editor
**************************************************************************/
public Object getEditorValue() {

Map result = new HashMap();

if(   name_input != null) result.put("name"   ,    name_input.getText());
if(  value_input != null) result.put("value"  ,   value_input.getText()); 
if(comment_input != null) result.put("comment", comment_input.getText());
if(   size_input != null) result.put("size"   ,    size_input.getText()); 

return result;

} // end of getEditorValue

public Component getEditorComponent() { return this; }


/***************************************************************************
* main method for testing 
***************************************************************************/
public static void main(String[] args) throws Exception {

DataField field1 = new SignedIntField("name", 32, 
                                      "This is the first field");

DataField field2 = new SignedIntField("longer_name", 32, 
                                  "I hereby add a second field to this thing");

field1.value(new Integer(10234));
field2.value(new Integer(4));

DataFieldRenderer renderer1 = new DataFieldRenderer(field1);
DataFieldRenderer renderer2 = new DataFieldRenderer(field2);

Dimension dimen = renderer1.getPreferredSize();
System.out.println("preferred size: "+ dimen.getWidth()
                               +" x "+ dimen.getHeight() );

JFrame frame = new JFrame();
frame.getContentPane().setLayout(new BoxLayout(frame.getContentPane(),BoxLayout.Y_AXIS));
frame.getContentPane().add(renderer1);
frame.getContentPane().add(renderer2);


frame.pack();
frame.setVisible(true);

} // end of main method






} // end of DataFieldRenderer class
