import javax.swing.tree.*;
import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.event.*;

/****************************************************************************
* This is the basic subclass for any parsed section of data. 
****************************************************************************/
public abstract class DataNode extends DefaultMutableTreeNode {

/*******
* data *
*******/
boolean structure_is_mutable;


/**************************************************************************
* constructor
**************************************************************************/
DataNode() { 

structure_is_mutable=true;

  } // end of constructor

/***********************************************************************
* mutibility accessors 
***********************************************************************/
public void setStructureIsMutable(boolean flag) { structure_is_mutable=flag; }
public boolean structureIsMutable() { return structure_is_mutable; }
public boolean canDelete() {

    DataNode parent =(DataNode)getParent();
    if(parent==null) return false;
    if(parent.structureIsMutable()) return true;
    else                          return false;

} // end of can delete method


/***********************************************************************
* abstract methods
***********************************************************************/
public abstract void  read(BitInputStream   in) throws IOException;
public abstract void write(BitOutputStream out) throws IOException;

public abstract int size();


/**************************************************************************
* method allowing each node to say how it should be displayed
**************************************************************************/
public Component displayComponent(boolean selected, boolean expanded, 
                                  int row, boolean hasFocus) {

    return new JLabel(toString() );

} // end of displayComponent method

/**************************************************************************
* the following methods allow each node to say how it should be edited.
**************************************************************************/
public DataNodeEditor getEditor(int row) {

    return null;

}

public void setUserObject(Object o) { 
System.out.println("DataNode.setUserObject: "+o);
}

/**************************************************************************
* factory method to create a data node corresponding to a given name.
* It first tries to create an object with the given class name.
* If this fails, it returns null
**************************************************************************/
static DataNode createByName(String name) {

try {
    DataNode node = (DataNode)Class.forName(name).newInstance();
    return node;

} catch(Exception e) {}

/************************************************************************
* if we got here, then we failed in making an object of the named class *
************************************************************************/
try {
return new DataBlock(name);
} catch(IOException e) {}

return null;

} // end of createByName factory method

/****************************************************************************
*
****************************************************************************/
DataNode find(String pattern, int depth) {

//System.out.println("matching "+pattern+" to "+this);

    /***************************
    * see if this node matches *
    ***************************/
    String string=toString();
    if(string!=null && string.indexOf(pattern) != -1) {
        return this;
    }

    /****************************************************
    * we didn't match, so try to match all our children *
    ****************************************************/
    --depth;
    if(depth==0) return null;
    for(int i=0; i<getChildCount(); ++i) {
        DataNode node = (DataNode)getChildAt(i);
        DataNode found = node.find(pattern, depth);
        if(found!=null) return found;
    }

    /***************************************
    * if we get here, then nothing matched *
    ***************************************/
    return null;
} // end of find method


} // end of DataNode abstract class
