import java.util.*;
import java.io.*;

/******************************************************************************
* This class represents an ordered list of directories which may be searched
* to find files.
******************************************************************************/
public class FileSearchPath {

/*******
* data *
*******/
List path;

/***************************************************************************
* constructor
***************************************************************************/
FileSearchPath() {

path = new ArrayList();

} // end of constructor

/***************************************************************************
* ...with single directory
***************************************************************************/
FileSearchPath(File dir) {

this();
add(dir);

} // end of constructor


/***************************************************************************
* add a directory to the end of the search path
***************************************************************************/
public void add(File dir) {

if( ! dir.isDirectory() ) return;

path.add(dir);

} // end of add method

/***************************************************************************
* find a file with a given name. Searches the directories in the path
* in order and returns the first file of that name found.
* If no file is found, returns null.
***************************************************************************/
public File find(String name) {

for(Iterator it= path.iterator(); it.hasNext(); ) {
    File dir = (File)it.next();

    File file = new File(dir, name);
    if(file.isFile()) return file;
}

return null;

} // end of find method

/***************************************************************************
* same as find(), except that it looks for sub-directories instead of files. 
***************************************************************************/
public File findDirectory(String name) {

for(Iterator it= path.iterator(); it.hasNext(); ) {
    File dir = (File)it.next();

    File file = new File(dir, name);
    if(file.isDirectory()) return file;
}

return null;

} // end of find method


/***************************************************************************
* handy string representation 
***************************************************************************/
public String toString() {

StringBuffer string = new StringBuffer();

for(Iterator it= path.iterator(); it.hasNext(); ) {
    File dir = (File)it.next();

    string.append(dir.toString());
    if(it.hasNext() ) string.append(":");
}

return string.toString();

} // end of toString method

/***************************************************************************
* static method to create a FileSearchPath corresponding to the current
* CLASSPATH 
***************************************************************************/
public static FileSearchPath getClassPath() {

StringTokenizer tokenizer = new StringTokenizer(
                                        System.getProperty("java.class.path"),
                                        System.getProperty("path.separator")  );

FileSearchPath path = new FileSearchPath();

while(tokenizer.hasMoreTokens()) {
    String token = tokenizer.nextToken();

    path.add(new File(token));

} // end of loop over path string tokens

return path;
    

} // end of getClassPath method

/***************************************************************************
* returns a List of all the files passing a given FilenameFilter
* which occur in any of the file directories. The file names are
* sorted and uniqued.
***************************************************************************/
SortedSet list(FilenameFilter filter) {

SortedSet set = new TreeSet();

for(Iterator it= path.iterator(); it.hasNext(); ) {
    File dir = (File)it.next();

    String[] files = dir.list(filter);
    for(int i=0; i< files.length; ++i) {

        set.add(files[i]);
    }
}

return set;

} // end of list method

/***************************************************************************
* returns the name of the first file found in the path which passes
* the given filter. Returns null if no files passed
***************************************************************************/
String listOne(FilenameFilter filter) {

SortedSet set = new TreeSet();

for(Iterator it= path.iterator(); it.hasNext(); ) {
    File dir = (File)it.next();

    String[] files = dir.list(filter);
    if(files.length>0) return files[0];
}

return null;

} // end of list method



    


/***************************************************************************
* main method for testing 
***************************************************************************/
public static void main(String[] args) throws Exception {

//System.out.println(System.getProperties());

FileSearchPath path = new FileSearchPath();

path.add(new File("/usr/home/pier") );
path.add(new File("/opt/local/") );
System.out.println(path);

System.out.println(getClassPath());

} // end of main method




} // end of FileSearchPath class
