import javax.swing.*;
import java.awt.*; 
import java.util.*;

public class GlobalLayout implements LayoutManager {

/**************
* class data *
*************/
static java.util.List max_widths = new ArrayList();
static int max_height=0;



/****************************************************************************
* constructor
****************************************************************************/
GlobalLayout() {

} // end of constructor

/****************************************************************************
* adjust the maximum width for a given cell given the width of a component 
* in that cell.
****************************************************************************/
private void setMaxWidth(int index, int width) {

if(width > getMaxWidth(index) ) {
    
    max_widths.set(index, new Integer(width) );
}


} // end of  setMaxWidth method

/****************************************************************************
* returns the maximum width of a given cell
****************************************************************************/
private int getMaxWidth(int index) {

try { return ((Integer)max_widths.get(index)).intValue(); }
catch(IndexOutOfBoundsException e) {
    max_widths.add(new Integer(0) );
    return 0; 
}

} // end of getMaxWidth method

/****************************************************************************
* returns the sum of the widths of all the cells
****************************************************************************/
private int totalWidth() {

int sum=0;

for(Iterator it = max_widths.iterator(); it.hasNext(); ) {
    Integer width = (Integer)it.next();

    sum += width.intValue();

} 

return sum;

} // end of totalWidth method


/****************************************************************************
* adjusts the maximum sizes  to include the given component
****************************************************************************/
private void adjustMaxSizes(int index, Component comp) {

try {
    JComponent jcomp = (JComponent)comp;
    Dimension dimen = jcomp.getPreferredSize();
    int height = (int)dimen.getHeight();
    int width  = (int)dimen.getWidth();

    if(height>max_height) max_height=height;
    setMaxWidth(index,width);

} catch(ClassCastException e) {}

} // end of adjustMaxSizes method


/****************************************************************************
* these do nothing since we don't associate strings with components
****************************************************************************/
public void addLayoutComponent(String name, Component comp) {} 
public void removeLayoutComponent(Component comp) {}



/****************************************************************************
* this is where all the work is done
****************************************************************************/
public void layoutContainer(Container parent) {

Component[] comp = parent.getComponents();

int left = 0;
for(int i=0; i<comp.length; ++i) {

    adjustMaxSizes(i, comp[i]);

    int width = getMaxWidth(i);
    comp[i].setBounds(left, 0, width, max_height );
    left += width;

} // end of loop over components


//parent.setSize(preferredLayoutSize(parent));
//try { ((JComponent)parent).setPreferredSize( preferredLayoutSize(parent) ); } 
//catch(ClassCastException e) {System.out.println("not A JComponent");}
    
} // end of layoutContainer method

/****************************************************************************
* 
****************************************************************************/
public Dimension minimumLayoutSize(Container parent) {

return new Dimension(totalWidth(), max_height);

} // end of minimumLayoutSize method

/****************************************************************************
* 
****************************************************************************/
public Dimension preferredLayoutSize(Container parent) {

return new Dimension(totalWidth(), max_height);

} // end of preferredLayoutSize method

/***************************************************************************
* main method for testing 
***************************************************************************/
public static void main(String[] args) throws Exception {

JPanel panel1 = new JPanel();
panel1.setLayout(new GlobalLayout() );

panel1.add(new JLabel("first field") );
panel1.add(new JLabel("second field") );
panel1.add(new JLabel("third field") );

System.out.println(panel1.getPreferredSize() );

JPanel panel2 = new JPanel();
panel2.setLayout(new GlobalLayout() );

panel2.add(new JLabel("first") );

JLabel two = new JLabel("second");
//two.setAlignmentX((float)1.0);
two.setOpaque(true);
two.setBackground(Color.red);
two.setHorizontalAlignment(SwingConstants.RIGHT);
panel2.add(two);

panel2.add(new JLabel("third") );


JFrame frame = new JFrame();
frame.getContentPane().setLayout(new BoxLayout(frame.getContentPane(),BoxLayout.Y_AXIS));
frame.getContentPane().add(panel1);
frame.getContentPane().add(panel2);


frame.pack();
frame.setVisible(true);


} // end of main method



} // end of GlobalLayout class
