import java.util.*;
import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

public class NewNodeSelector extends JDialog implements ActionListener,
                                                 ListSelectionListener{

/*******
* data *
*******/
JTextField name;
JButton ok;
boolean got_value;

//APIDKnower apids;

/***************************************************************************
* constructor 
***************************************************************************/
NewNodeSelector(JFrame frame)  {

super(frame,"New Node Chooser", false);

//try { apids = ((PacketEditor)frame).getAPIDs(); }
//catch(ClassCastException e) { apids = new APIDKnower(); }

/**************************************
* panel to hold textfield and buttons *
**************************************/
JPanel bottom = new JPanel();
bottom.setLayout(new BoxLayout(bottom,BoxLayout.Y_AXIS) );

/*******************
* text input field *
*******************/
name = new JTextField(20);
name.addActionListener(this);
bottom.add(name);

/************************
* panel to hold buttons *
************************/
JPanel buttons = new JPanel();
ok = new JButton("OK");
ok.addActionListener(this);
buttons.add(ok);

bottom.add(buttons);

getContentPane().add(bottom,BorderLayout.SOUTH);


/**********************************
* panel containing browsing lists *
**********************************/
JPanel lists = new JPanel();
lists.setLayout(new BoxLayout(lists, BoxLayout.X_AXIS) );

/********
* APIDs *
********/
//lists.add(makeListBox("APIDs", apids.list()) );

/********************
* named data blocks *
********************/
Collection data_blocks = DataBlock.getFileSearchPath()
                                 .list(new FilenameSuffixFilter(".db"));

lists.add(makeListBox("Data Blocks", data_blocks));

/**********
* classes *
**********/
try {

Class node_class = Class.forName("DataNode");

Collection classes = FileSearchPath.getClassPath().
                    list(new ClassFilenameFilter(node_class) );

lists.add(makeListBox("Data Structures", classes));

} catch(ClassNotFoundException e) {}





getContentPane().add(lists, BorderLayout.CENTER);

pack();
setVisible(false);

} // end of constructor

/***************************************************************************
* utility method for making a list box 
***************************************************************************/
private JComponent makeListBox(String label, Collection items) {

/**********************************
* make a panel to hold everything *
**********************************/
JPanel panel = new JPanel();
panel.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS));

/********
* label *
********/
panel.add(new JLabel(label+":"));

/************************************
* clip off the file name extensions *
************************************/
String[] strings = new String[items.size()];
int i=0;
for(Iterator it= items.iterator(); it.hasNext(); ) {
    String name = (String)it.next();

    int index = name.indexOf('.');
    if(index!= -1) name = name.substring(0,index);
    strings[i++] = name;
}

JList list = new JList(strings);
list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
list.addListSelectionListener(this);
list.addMouseListener(new DoubleClickListener() );

JScrollPane scroll = new JScrollPane(list);
scroll.setPreferredSize(new Dimension(150,200));

panel.add(scroll);

return panel;

} // end of makeListBox method


/***************************************************************************
* accessors 
***************************************************************************/
public DataNode getDataNode() { 

/*****************************************
* get the text string in the input field *
*****************************************/
String text = name.getText();

System.out.println("text=|"+text+"|");

/***********************************
* try interpreting this as an APID *
***********************************/
/*
int index = text.indexOf(' ');
if(index != -1) {
    try { 
        int apid = Integer.parseInt(text.substring(0,index));
System.out.println("apid="+apid);
        return apids.makePacket(apid);
   
    } 
    catch(NumberFormatException e) { System.out.println("number format ex"); e.printStackTrace();
    }
    catch(IOException e) {System.out.println("IO exception");}
}

System.out.println("wasn't an APID");
*/

/************************************************************
* next try interpreting as a class name or a datablock name *
************************************************************/
DataNode node= DataNode.createByName(text );
if(node != null) return node;

System.out.println("wasn't a known class or db");

/**************************************************
* there's no known class or database by that name *
**************************************************/
return null; 

} // end of getDataNode method

/***************************************************************************
* respond to various actions 
***************************************************************************/
public void actionPerformed(ActionEvent e) {

System.out.println("got action");

if(e.getSource() == name) {
    /************************
    * event from name field *
    ************************/
    System.out.println(name.getText());
    close(true);
} else if(e.getSource() == ok ) {
    /********************
    * OK button pressed *
    ********************/
    System.out.println("OK");
    close(true);
}

} // end of actionPerformed method

/***************************************************************************
* respond to JList selection actions
***************************************************************************/
public void valueChanged(ListSelectionEvent e) {

//if(e.getValueIsAdjusting() ) return;


System.out.println("list selection value changed");

JList list = (JList)e.getSource();
int index = list.getSelectedIndex();

System.out.println("index="+index+" value=|"+
                   (String)list.getModel().getElementAt(index)+"|");

/*****************************************
* put the name into the text input field *
*****************************************/
name.setText((String)list.getModel().getElementAt(index) );

} // end of valueChanged method


/***************************************************************************
* be done
***************************************************************************/
public synchronized void close(boolean got_value) {

this.got_value=got_value;

System.out.println("closing");
setVisible(false);
notifyAll();

} // end of close method

    
/***************************************************************************
* pop up the dialog box, and wait for a result.
* returns true if a valid result was choosen, and false if the selection
* was canceled
***************************************************************************/
public synchronized boolean prompt() {

/*********************
* pop up the chooser *
*********************/
setVisible(true);

/*****************************************************************
* wait until get get some signal that the user is done inputting *
*****************************************************************/
try { wait(); }
catch(InterruptedException e) { System.out.println("interupted"); }

System.out.println("now I'm done");



return got_value;

} // end of prompt method

/****************************************************************************
*****************************************************************************
* embedded class for detecting double clicks on the list windows
****************************************************************************/
public class DoubleClickListener extends MouseAdapter {

    public void mouseClicked(MouseEvent e) {
        if(e.getClickCount()==2) close(true);
    } // end of mouseClicked method 

} // end of DoubleClickListener embedded class


/***************************************************************************
* main method for testing 
***************************************************************************/
public static void main(String[] args) throws Exception {

JFrame frame = new JFrame();

frame.setVisible(true);

NewNodeSelector selector = new NewNodeSelector(frame);

if(selector.prompt() ) {

    System.out.println(selector.getDataNode() );

} 



} // end of main method



} // end of NewNode Selector
