import java.awt.*;
import javax.swing.*;
import java.io.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.tree.*;
import java.awt.print.*;
import java.awt.geom.*;

public class PacketEditor extends JFrame implements ActionListener {

/*******
* data *
*******/
JFileChooser file_chooser;
SearchDialog search_dialog;
NewNodeSelector node_selector;
JMenuBar menu_bar;
JComponent display;
Dimension natural_size;
File current_file;
JTree tree;

DataStream stream;
Class stream_type;

BackgroundSearcher searcher;


//APIDKnower apids;

/***************************************************************************
* constructor
***************************************************************************/
PacketEditor() {


/***********************************
* set the look and feel of the GUI *
***********************************/
try {
//    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());

} catch (Exception e) {
  //  System.out.println("exception setting look and feel");
}

addWindowListener(new Closer() );

natural_size = new Dimension(600,800);

/***********
* menu bar *
***********/
menu_bar = new JMenuBar();
setJMenuBar(menu_bar);

/************
* file menu *
************/
JMenu menu = new JMenu("File");
menu_bar.add(menu);

JMenuItem menuItem = new JMenuItem("Open");
menuItem.addActionListener(this);
menu.add(menuItem);

menuItem = new JMenuItem("Save");
menuItem.addActionListener(this);
menu.add(menuItem);

menuItem = new JMenuItem("New Stream");
menuItem.addActionListener(this);
menu.add(menuItem);

menuItem = new JMenuItem("Print");
menuItem.addActionListener(this);
menu.add(menuItem);



menuItem = new JMenuItem("Exit");
menuItem.addActionListener(this);
menu.add(menuItem);

/************
* edit menu *
************/
menu = new JMenu("Edit");
menu_bar.add(menu);

menuItem = new JMenuItem("Add", KeyEvent.VK_T);
menuItem.addActionListener(this);
menu.add(menuItem);

menuItem = new JMenuItem("Delete", KeyEvent.VK_T);
menuItem.addActionListener(this);
menu.add(menuItem);

/**************
* format menu *
**************/
menu = new JMenu("Format");
menu_bar.add(menu);

ActionListener listener = new FormatListener();

for(Iterator it = FileSearchPath.getClassPath()
                  .list(new ClassFilenameFilter(DataStream.class))
                  .iterator();
    it.hasNext(); ) {

    String name = (String)it.next();
    name = name.substring(0,name.indexOf(".class"));

    menuItem = new JMenuItem(name, KeyEvent.VK_T);
    menuItem.addActionListener(listener);
    menu.add(menuItem);

}


/**************
* window menu *
**************/
menu = new JMenu("Window");
menu_bar.add(menu);

menuItem = new JMenuItem("Search", KeyEvent.VK_FIND);
menuItem.addActionListener(this);
menu.add(menuItem);


menuItem = new JMenuItem("Page Up", KeyEvent.VK_PAGE_UP);
menuItem.addActionListener(this);
menu.add(menuItem);

menuItem = new JMenuItem("Page Down", KeyEvent.VK_PAGE_DOWN);
menuItem.addActionListener(this);
menu.add(menuItem);

menuItem = new JMenuItem("Rewindow");
menuItem.addActionListener(this);
menu.add(menuItem);



/**************************************************
* initialize the apid list do a default empty one *
**************************************************/
//apids = new APIDKnower();

/********************************
* initialize to display no file *
********************************/
setStreamType();
displayNoFile();


/*************
* make it go *
*************/
pack();
setVisible(true);


} // end of constructor


/*****************************************************************************
* set the type of stream that will be used the next time we read a file.
* The argument is a string giving the name of a DataStream subclass
*****************************************************************************/
private void setStreamType(String type) {

try { stream_type = Class.forName(type); }
catch(ClassNotFoundException e) {System.out.println(e);}

/**************************
* reopen the current file *
**************************/
if(current_file != null) openFile(current_file);
else                     displayNoFile();

} // end of setStreamType method


/*****************************************************************************
* sets the stream type to the default, which is the first stream to be listed
* in the CLASSPATH
*****************************************************************************/
private void setStreamType() {

String name = FileSearchPath.getClassPath()
                           .listOne(new ClassFilenameFilter(DataStream.class));

name = name.substring(0,name.indexOf(".class"));

setStreamType(name);

} // end of setStreamType to default method





/*****************************************************************************
* utility method to make an instance of a datastream of the current type
*****************************************************************************/
private DataStream makeStream() {

if(stream_type==null) return null;

try {return (DataStream)stream_type.newInstance(); } 
catch(Exception e) {System.out.println(e); return null; }

} // end of makeStream method



/***************************************************************************
* open a particular file  The actual reading of the file is done in
* the background (i.e. in a separate thread so that the progress monitor will
* work properly
***************************************************************************/
public void openFile(File file) {

new BackgroundReader(this, file).start();

} // end of open file method

/***************************************************************************
* display nothing  
***************************************************************************/
public void displayNoFile() {

//packets = new PacketStream(apids);
stream = makeStream();
setUpTree();

/*******************
* set window title *
*******************/
setTitle("xing:");

/********************************
* mark the current file as null *
********************************/
current_file=null;

} // end of displayNoFile method

/**************************************************************************
* opens a file chosen interactively by the user
**************************************************************************/
public void openFile() {

/****************************************************
* create the file chooser if it hasn't been already *
****************************************************/
if(file_chooser==null) {
    file_chooser = new JFileChooser(System.getProperty("user.dir") ); 
}


/**********************************
* get a file from the file choser *
**********************************/
if(file_chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
    /********
    * do it *
    ********/
    openFile(file_chooser.getSelectedFile() );
} 

} // end of openFile method

/*************************************************************************
* save the packet stream to a given file
*************************************************************************/
public void save(File file) {

try {
    BitOutputStream out = new BitOutputStream(new FileOutputStream(file));
    stream.write(out);
} catch(IOException e) {
    JOptionPane.showMessageDialog(this, "Couldn't save "+file+
                                        " "+e.getMessage()     );
}

} // end of save method


/*************************************************************************
* write the current file to a file chosen via the file chooser
*************************************************************************/
public void saveFile() {

/****************************************************
* create the file chooser if it hasn't been already *
****************************************************/
if(file_chooser==null) {
    file_chooser = new JFileChooser(System.getProperty("user.dir") ); 
}


/**********************************
* get a file from the file choser *
**********************************/
if(file_chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
    /********
    * do it *
    ********/
    save(file_chooser.getSelectedFile() );
} 

} // end of save method


/*************************************************************************
* respond to actions from the menu bar
*************************************************************************/
public void actionPerformed(ActionEvent e) {

    String command = e.getActionCommand();

    if(     command.equals("Open")   ) openFile();
    else if(command.equals("Save")   ) saveFile();
    else if(command.equals("New Stream")) displayNoFile();
    else if(command.equals("Print")) new BackgroundPrinter().start();
    else if(command.equals("Exit")   ) System.exit(0);

    else if(command.equals("Search")) popUpSearch();
    else if(command.equals("Page Up"  )) pageUp();
    else if(command.equals("Page Down")) pageDown();
    else if(command.equals("Rewindow")) rewindow();


    else if(command.equals("Delete") ) {

        TreePath[] paths = tree.getSelectionPaths();
        for(int i=0; i<paths.length; ++i) {
            DataNode node = (DataNode)paths[i].getLastPathComponent();
            System.out.println("removing "+node);

            DataNode parent = (DataNode)node.getParent();
            System.out.println("parent has "+parent.getChildCount()+" children");

            if(node.canDelete()) {
                ((DefaultTreeModel)tree.getModel()).removeNodeFromParent(node);
            }

            System.out.println("nodes's parent now "+node.getParent() );
            System.out.println("parent has "+parent.getChildCount()+" children");
            
        }

    } else if(command.equals("Add") ) {
        new BackgroundAdder(this).start();

    } 
        

} // end of actionPerformed

/**************************************************************************
* utility method to set up the tree and display it
**************************************************************************/
private void setUpTree() {

/*************************
* remove any old display *
*************************/
if(display != null) getContentPane().remove(display);

/****************************
* create and configure tree *
****************************/
tree = new JTree(stream);

Renderer renderer = new Renderer(tree);

tree.setCellRenderer(renderer);
tree.setCellEditor(renderer);
tree.setEditable(true);

tree.setShowsRootHandles(true);
tree.setRootVisible(true);

tree.getSelectionModel().setSelectionMode(
                                TreeSelectionModel.CONTIGUOUS_TREE_SELECTION);

/****************************
* add the tree to the frame *
****************************/
JScrollPane scroll = new JScrollPane(tree);
scroll.setPreferredSize(natural_size);
getContentPane().add(scroll, BorderLayout.CENTER);
display=scroll;

pack();

} // end of SetUpTree method


/***************************************************************************
* move the parsed data window down by the width of the window.
***************************************************************************/
private void pageDown() {

System.out.println("pageDown");

int first = stream.getFirstParsed();
int last  = stream.getLastParsed();

if(last<0) return;


int nparsed  = stream.getParsedCount();
int at_end   = stream.getEndUnparsedCount();

int shift =nparsed;
if(shift>at_end) shift=at_end;

stream.rewindow(first+shift, last+shift);

/************************************
* force the changes to be displayed *
************************************/
DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
model.reload();

} // end of pageDown method

/***************************************************************************
* move the parsed data window up by the width of the window.
***************************************************************************/
private void pageUp() {

int first = stream.getFirstParsed();
int nparsed  = stream.getParsedCount();

first -= nparsed;
if(first<1) first=1;

stream.rewindow(first, first+nparsed-1);

/************************************
* force the changes to be displayed *
************************************/
DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
model.reload();

} // end of pageDown method

/***************************************************************************
* 
***************************************************************************/
private void rewindow() {

    /***************************************
    * pop up a dialog to get the new range *
    ***************************************/
    int last = stream.getLastParsed();
    if(last==-1) last=stream.getCount();

    String current = stream.getFirstParsed()+"-"+last;

    String range = JOptionPane.showInputDialog(
                                   "Enter new range (currently "+current+"):");
    if(range==null) return;

    /*************************
    * parse the range string *
    *************************/
    int from =1;
    int to = stream.getCount();

    int index = range.indexOf('-');

    if(index >=0) {
        try { from = Integer.parseInt(range.substring(0,index).trim() ); }
        catch(NumberFormatException e) {}

        try { to = Integer.parseInt(range.substring(index+1).trim()); }
        catch(NumberFormatException e) {}
    } else {
        try { to = Integer.parseInt(range.trim()); }
        catch(NumberFormatException e) {}
    }

    /**********************
    * rewindow the stream *
    **********************/
    stream.rewindow(from,to);

    DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
    model.reload();

} // end of rewindow method

    

/***************************************************************************
* 
***************************************************************************/
private void popUpSearch() {

    if(search_dialog==null) search_dialog = new SearchDialog(this);
    search_dialog.show();
 //   search_dialog.doLayout();
}


/***************************************************************************
* Start a new search thread if one is not already running
***************************************************************************/
public void search() { 

    if(searcher != null && searcher.isAlive()) return;

    searcher = new BackgroundSearcher();
    searcher.start(); 

} // end of search method



/*************************************************************************
**************************************************************************
* inner class for handling format menu selections
*************************************************************************/
public class FormatListener implements ActionListener {

public void actionPerformed(ActionEvent e) {

    setStreamType(e.getActionCommand());

} // end of actionPerformed method

} // end of FormatListener inner class ***********************************



/*************************************************************************
**************************************************************************
* respond to window events to quit on close
*************************************************************************/
public class Closer extends WindowAdapter {

    public void windowClosing(WindowEvent e) {

        System.exit(0);
    }

} // end of Closer embedded class

/*************************************************************************
**************************************************************************
* ebedded class for reading a file in the background
*************************************************************************/
class BackgroundReader extends Thread {

File file;
JFrame frame;

BackgroundReader(JFrame frame, File file) { 
    this.frame=frame;
    this.file=file;
}

public void run() { 

/************************************
* create and read the packet stream *
************************************/
DataStream old_stream=stream;
stream = makeStream();


/****************************************************
* see if there are any decorations on the file name *
****************************************************/
File full_file = file;
String filename = file.toString();

int index = filename.indexOf(' ');
if(index>=0) {
    /*************************
    * there are decorations *
    ************************/
    int from =1;
    int to = -1;

    /*********************************************
    * separate the filename from the decorations *
    *********************************************/
    file = new File(filename.substring(0, index).trim());
    String decorations = filename.substring(index);

    // System.out.println("decorations="+decorations);

    /******************
    * look for a dash *
    ******************/
    index = decorations.indexOf('-');

    if(index >=0) {
        try { from = Integer.parseInt(decorations.substring(0,index).trim() ); }
        catch(NumberFormatException e) {}

        try { to = Integer.parseInt(decorations.substring(index+1).trim()); }
        catch(NumberFormatException e) {}
    } else {
        try { to = Integer.parseInt(decorations.trim()); }
        catch(NumberFormatException e) {}
    }

    // System.out.println("from="+from+" to="+to);

    /************************
    * set the parsed limits *
    ************************/
    stream.setFirstParsedPacket(from);
    stream.setLastParsedPacket(to);


} // end if there were decorations on the file name




try {
    /**************************************************
    * open the file with an attached progress monitor *
    **************************************************/
    BitInputStream in = new BitInputStream(
                        new ProgressMonitorInputStream(frame, "Reading " + file,
                        new BufferedInputStream(
                        new FileInputStream(file))) );
    stream.read(in);
    in.close();
    old_stream=null;

} catch(IOException e) {
    /***********************************************
    * couldn't read the file, so show a dialog and
    * keep the file the same
    ************************************************/
    JOptionPane.showMessageDialog(frame, "Couldn't open "+file+
                                        " "+ e.getMessage() );
    stream=old_stream;
    return; 
}

/*************************
* remove any old display *
*************************/
if(display != null) getContentPane().remove(display);

/****************************
* create and configure tree *
****************************/
tree = new JTree(stream);

Renderer renderer = new Renderer(tree);

tree.setCellRenderer(renderer);
tree.setCellEditor(renderer);
tree.setEditable(true);

tree.getSelectionModel().setSelectionMode(
                                TreeSelectionModel.CONTIGUOUS_TREE_SELECTION);

/****************************
* add the tree to the frame *
****************************/
JScrollPane scroll = new JScrollPane(tree);
scroll.setPreferredSize(natural_size);
getContentPane().add(scroll, BorderLayout.CENTER);
display=scroll;

pack();

/****************************************
* add the file name to the window title *
****************************************/
frame.setTitle("xing: "+file);


/*****************************
* save the current file name *
*****************************/
current_file=full_file;

} // end of run method


} // end of BackgroundReader embedded class

/*************************************************************************
**************************************************************************
* ebedded class for adding a node in a different thread
* We apparently have to do this is a different thread since we are in the 
* event thread when we want to pop up the chooser dialog, so we have to free
* up the event thread to handle events in the popup window. Go figure...
*************************************************************************/
class BackgroundAdder extends Thread {

private JFrame frame;

BackgroundAdder(JFrame frame) { this.frame = frame; }

public void run() {

        if(node_selector == null) node_selector = new NewNodeSelector(frame);
        if(node_selector.prompt() ) {
            /*****************************
            * the selector got a new node *
            ******************************/
            DataNode node = node_selector.getDataNode();
            if(node == null) return;

            /***************************************
            * figure out where to put the new node *
            ***************************************/
            try {
                DataNode selected=(DataNode)tree.getLastSelectedPathComponent();
                DataNode parent = (DataNode)selected.getParent();
                int index = parent.getIndex(selected) + 1;

                DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
                model.insertNodeInto(node, parent,index) ;
            } catch(NullPointerException e) {
                /********************************************************
                * if anything screws up add to the end of the root node *
                ********************************************************/
                DefaultTreeModel model = (DefaultTreeModel)tree.getModel();

                model.insertNodeInto(node, stream, stream.getChildCount());
            }

                

        }

} // end of run method

} // end of  BackgroundAdder embedded class


/*************************************************************************
**************************************************************************
* inner class for reading a file in the background
*************************************************************************/
class BackgroundSearcher extends Thread {

public void run() {

    System.out.println("Searching...");

    /**********************************************
    * don't bother if there's nothing in the tree *
    **********************************************/
    if(stream.getChildCount() == 0) return;

    /********************************************
    * don't bother searching for a blank string *
    ********************************************/
    String pattern = search_dialog.getPattern();
    if(pattern==null || pattern.equals("")) return;

    /*****************************************
    * find the starting place for the search *
    *****************************************/
    DataNode start=null;
    if(!search_dialog.fromBeginning()) {
        try { 
            start = (DataNode)tree.getSelectionPath().getLastPathComponent();
            while(stream.getIndex(start)==-1) {
                start = (DataNode)start.getParent();
            }

        } 
        catch(NullPointerException e) {}
    }
    

    /****************
    * do the search *
    ****************/
    DataNode node = stream.find(pattern, search_dialog.justWindow(), 
                                start, search_dialog.getDepth(), search_dialog);
    search_dialog.resetProgress();

    if(node==null) {
        System.out.println("Could not find "+pattern);
        return;
    }

    /******************************
    * indicate the search results *
    ******************************/
    DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
    if(!search_dialog.justWindow() ) model.reload();

    TreePath path = new TreePath(node.getPath());
    tree.scrollPathToVisible(path);

 //   tree.makeVisible(path);
    int row = tree.getRowForPath(path);
    tree.setSelectionRow(row);


} // end of run method

} // end of BackgroundSearcher inner class

/*************************************************************************
**************************************************************************
* inner class for printing in the background
*************************************************************************/
public class BackgroundPrinter extends Thread implements Printable {

int last_page;
double height_printed;

/**************************************************************************
* starts a new print job
**************************************************************************/
public void run() {

    /*********************************
    * initialize the page accounting *
    *********************************/
    last_page=0;
    height_printed=0.0;

    /*************************
    * create the printer job *
    *************************/
    PrinterJob job = PrinterJob.getPrinterJob();
    job.setPrintable(this);

    /***************************************************************
    * pop up a dialog and quit if the job was canceled via the GUI *
    ***************************************************************/
    if( ! job.printDialog()) return;

    /****************************
    * start the actual printing *
    ****************************/
    try { job.print(); }
    catch(PrinterException e) {
        System.err.println(e);
    }

} // end of run method

/***********************************************************************
* the print job calls this method to render a page or a part of a page
***********************************************************************/
public int print(Graphics g, PageFormat format, int page) {
    Graphics2D g2 = (Graphics2D)g;

    System.out.println("page "+page);

    /***************************
    * get the size of the tree *
    ***************************/
    Dimension dimen = tree.getSize();
    double width = dimen.getWidth();
    double height = dimen.getHeight();

    /****************************
    * check if this page exists *
    ****************************/
    if(height_printed >= height) return NO_SUCH_PAGE;

    /***********************************************
    * scale the tree to fit in the page boundaries *
    ***********************************************/
    double scale = format.getImageableWidth() / width;

    g2.scale(scale, scale);
    g2.translate(format.getImageableX(), format.getImageableY());
    g2.translate(0, -height_printed);

    /*******************************************************
    * calculate the height of one page in tree coordinates *
    *******************************************************/
    double page_height = format.getImageableHeight()/scale;
    page_height -= 10.0;


    /********
    * print *
    ********/
    tree.print(g);

    /***********************************
    * update the current page location *
    ***********************************/
    if(page!= last_page) height_printed += page_height;
    last_page=page;

    return PAGE_EXISTS;


}

} // end of BackgroundPrinter inner class


/***************************************************************************
* main method  
***************************************************************************/
public static void main(String[] args) throws Exception {

/*****************************************************************************
* set the DataBlock file search path to look first in the current directory
* then in the local config directory and finally in the instalation directory
*****************************************************************************/
DataBlock.setFileSearchPath(FileSearchPath.getClassPath());



/********************
* create the editor *
********************/
PacketEditor ed = new PacketEditor();


/*********************************************
* load the command line file if there is one *
*********************************************/
if(args.length>=1) {

    /*********************************************************************
    * concatenate all the arguments to allow decorations on the filename *
    *********************************************************************/
    String arg = new String();
    for(int i=0; i<args.length; ++i) {
        arg = arg + args[i];
        if(i<args.length-1) arg = arg + " ";
    }

    ed.openFile(new File(arg) ); 

} // end if opening file from command line


} // end of main


} // end of PacketEditor class
