import javax.swing.tree.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.util.*;

class Renderer implements TreeCellRenderer, TreeCellEditor, 
                          ActionListener, TreeSelectionListener {

java.util.List listeners;
DataNodeEditor editor;

/*****************************************************************************
* constructor
*****************************************************************************/
Renderer(JTree tree) {

listeners = new ArrayList();

tree.addTreeSelectionListener(this);

} // end of constructor


/*****************************************************************************
* This is the method required by TreeCellRenderer. It tries to query the
* node to find out how to display itself
*****************************************************************************/
public Component getTreeCellRendererComponent(JTree tree, Object value,
                                       boolean selected, boolean expanded, 
                                       boolean leaf, int row,
                                       boolean hasFocus) {

//System.out.println("getTreeCellRendererComponent:");

    try {
        DataNode node = (DataNode)value;
        Component display = node.displayComponent(selected, expanded, row, 
                                                  hasFocus);
        if(selected) {
            try { ((JComponent)display).setOpaque(true); }
            catch(ClassCastException e) {}

            display.setBackground(Color.yellow);
        }
           

        return display;

    } catch(ClassCastException e) {
        return new JLabel("?????");
    }


} // end of getTreeCellRendererComponent method


/*****************************************************************************
* This is the method required by TreeCellEditor. It tries to query the
* node to find out how to edit itself
*****************************************************************************/
public Component getTreeCellEditorComponent(JTree tree,
                                            Object value,
                                            boolean isSelected,
                                            boolean expanded,
                                            boolean leaf,
                                            int row) {
    /*******************************
    * by default there's no editor *
    *******************************/
    editor=null;

    try {
        DataNode node = (DataNode)value;
        editor = node.getEditor(row);
        if(editor != null) return editor.getEditorComponent();

    } catch(ClassCastException e) { }

    return null; 

} // end of getTreeCellEditorComponent method

/*****************************************************************************
*
*****************************************************************************/
public void addCellEditorListener(CellEditorListener l) {

//System.out.println("addCellEditorListener: "+l);
listeners.add(l);

} // end of addCellEditorListener method

/*****************************************************************************
*
*****************************************************************************/
public void removeCellEditorListener(CellEditorListener l) {

System.out.println("removeCellEditorListener: "+l);

listeners.remove(l);
} // end of removeCellEditorListener method



/*****************************************************************************
* abort and ignore new value
*****************************************************************************/
public void cancelCellEditing() {
System.out.println("cancelCellEditing");

fireEditingCanceled();
cleanup();

} // end of cancelCellEditing method

/*****************************************************************************
* quit and save value
*****************************************************************************/
public boolean stopCellEditing() {
System.out.println("stopCellEditing");

fireEditingStopped();
cleanup();

return true;
}


/*****************************************************************************
* utility called by cancelCellEditing and stopCellEditing
*****************************************************************************/
private void cleanup() {

if(editor==null) return;

editor.removeActionListener(this);

} // end of cleanup method




/*****************************************************************************
*
*****************************************************************************/
public Object getCellEditorValue() {

System.out.println("getCellEditorValue");

if(editor != null) return editor.getEditorValue();
else               return null;

} // end of  getCellEditorValue method


/*****************************************************************************
*
*****************************************************************************/
public boolean isCellEditable(EventObject anEvent) {

if(editor==null) return false;

try {
    MouseEvent event = (MouseEvent)anEvent;
   // System.out.println("isCellEditable: "+event.getClickCount()+" clicks");

    if(event.getClickCount() == 2) {
        editor.addActionListener(this);
        return true;
    }

} catch (ClassCastException e) {}

return false;



} // end of  isCellEditable method



/*****************************************************************************
*
*****************************************************************************/
public boolean shouldSelectCell(EventObject anEvent) {

if(editor==null) return false;

System.out.println("shouldSelectCell:");

return true;
}


/*****************************************************************************
* this implements action listener
*****************************************************************************/
public void actionPerformed(ActionEvent e) {

System.out.println("ActionPerformed:");

stopCellEditing();

} // end of actionPerformed method

/*****************************************************************************
* this implements TreeSelectionListener
*****************************************************************************/
public void valueChanged(TreeSelectionEvent e) {

//System.out.println("valueChanged: "+e);

} // end of valueChanged method

/*****************************************************************************
* send an editing stopped event to all the listeners
*****************************************************************************/
public void fireEditingStopped() {

    System.out.println("fireEditingStopped");

    ChangeEvent e = new ChangeEvent(this);

    for(Iterator it= listeners.iterator(); it.hasNext(); ) {
        
        CellEditorListener l = (CellEditorListener)it.next();
        l.editingStopped(e);
    }
} // end of fireEditingStopped method 

/*****************************************************************************
* send an editing stopped event to all the listeners
*****************************************************************************/
public void fireEditingCanceled() {

    System.out.println("fireEditingCanceled");

    ChangeEvent e = new ChangeEvent(this);

    for(Iterator it= listeners.iterator(); it.hasNext(); ) {
        
        CellEditorListener l = (CellEditorListener)it.next();
        l.editingCanceled(e);
    }
} // end of fireEditingCanceled method 


} // end of Renderer class
