import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.*;


class TextBlock extends RawDataBlock {

/*****************************************************************************
* constructor
*****************************************************************************/
public TextBlock( int nbytes ) {

    this(new byte[nbytes]);

} // end of constructor

/*****************************************************************************
* constructor. If set_preferred=true, sets the preferred size to the
* current size.
*****************************************************************************/
public TextBlock(int nbytes, boolean set_preferred) {

    this(nbytes);
    if(set_preferred) setPreferredSize(nbytes);
}


/*****************************************************************************
* constructor from an existing array - just uses that array by reference
*****************************************************************************/
public TextBlock( byte[] data ) {

    super(data);


}

/*****************************************************************************
* 
*****************************************************************************/
protected DataNode makeDisplay() { return new TextBlockDisplay(); }



/****************************************************************************
*****************************************************************************
* embedded class for displaying data
****************************************************************************/
class TextBlockDisplay extends DataNode implements DataNodeEditor, KeyListener {

    /*******
    * data *
    *******/
    static final int columns = 32;
    JTextArea text;
    JTextArea editor;
    java.util.List listeners;

    /************************************************************
    * constructor 
    ************************************************************/
    TextBlockDisplay() {

        listeners = new ArrayList();

    } // end of constructor

    /**************************************************************
    * return component used for drawing
    **************************************************************/
    public Component displayComponent(boolean selected, boolean expanded, 
                                  int row, boolean hasFocus) {

        if(text == null) {
            /************************************
            * we need to create the text area
            ************************************/
            int ncols = 80;
            int nrows = size/ncols+1;
            text = new JTextArea(nrows, ncols);
            text.setLineWrap(true);
            text.setFont(new Font("Courier", Font.PLAIN, 12) );

            try { text.append(new String(data,"ASCII")); }
            catch(UnsupportedEncodingException e) { System.out.println(e); }

        } // end if we need to create the text area

        return text;

    } // end of displayComponent method

    /***************************************************************************
    * editing methods 
    ***************************************************************************/
    public DataNodeEditor getEditor(int row) { return this; }

    /***************************************************************************
    * allow a new value to be set  - called by tree after editing
    ***************************************************************************/
    public void setUserObject(Object o) {

        /**********************************
        * force the display to get remade *
        **********************************/
        text=null;

        
        try { data = o.toString().getBytes("ASCII"); }
        catch(UnsupportedEncodingException e) { System.out.println(e); }

        size = data.length;

    } // end of setUserObject method


    /*********************************************************************
    * the following methods implement the DataNodeEditor interface
    *********************************************************************/
    public Object getEditorValue() { return editor.getText(); }
    public Component getEditorComponent() { 

        editor = new JTextArea(text.getText());
        editor.setFont(new Font("Courier", Font.PLAIN, 12) );
        editor.setBorder(BorderFactory.createLineBorder(Color.black));
        editor.addKeyListener(this);
        return editor;

    }

    public void    addActionListener(ActionListener l) {listeners.add(l);}
    public void removeActionListener(ActionListener l) {listeners.remove(l);}

    /*****************************************************************
    * the following methods implement KeyListener so that we can
    * listen to the editor text area to intercept  returns
    *****************************************************************/
    public void keyReleased(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {}
    public void keyPressed(KeyEvent e) {

        if(e.getKeyChar() == '\n' ) {
            /***********************************************************
            * hitting return means we're done editing 
            * so fire off an action event to anyone listening
            * note we have to make a copy of the listener list first 
            * so that the listeners may remove themselves while
            * we are looping
            **********************************************************/
            java.util.List tmp = new ArrayList(listeners);
            for(Iterator it = tmp.iterator(); it.hasNext(); ) {
                ActionListener l = (ActionListener)it.next();
                l.actionPerformed(new ActionEvent(this,0,"Done Editing") );
            } // end of loop over listeners
        } // end if return was pressed

    } // end of keyPressed method


    /************************************************************
    * the following methods must be defined for a DataNode
    * we define them to represent an empty block of data
    * Since this is just a display which doesn't actually hold
    * and data
    ************************************************************/
    public void  read(BitInputStream   in) throws IOException {}
    public void write(BitOutputStream out) throws IOException {}

    public int size() { return 0; }

} // end of DataBlockDisplay embedded class



} // end of TextBlock class
