import java.io.*;

class UnsignedIntField extends AbstractDataField {

/**********************************************************************
* constructor 
**********************************************************************/
UnsignedIntField() { this(null,0,null); }
UnsignedIntField(String name, int size, String comment) {

super(name,size,comment);

/**********************
* min and max values *
*********************/
min_value = new Integer(0);

if(     size<32) max_value = new Integer(Integer.MAX_VALUE >> (31-size) );
else if(size<64) max_value = new    Long(   Long.MAX_VALUE >> (63-size) );

} // end of constructor


/**********************************************************************
* check if a given Number is a valid value for this field 
**********************************************************************/
public boolean isInValidRange(Number n) {

if(min_value != null && n.longValue() < min_value.longValue() ) return false;
if(max_value != null && n.longValue() > max_value.longValue() ) return false;

return true;

} // end of isInValidRange method

/**********************************************************************
* return a Number of the appropriate type for this field
* returns null if the data type of this field is unknown or weird.
**********************************************************************/
public Number validateType(Number n) {

if(size <  32) return new Integer(n.intValue()  );
else           return new    Long(n.longValue() );

} // end of validateType method

/**********************************************************************
* method to read this data field 
**********************************************************************/
public void read(BitInputStream in) throws IOException {

value=null;

/**************************************
* try to read using a built-in method *
**************************************/
if(in.isByteAligned()) {
    if(     size==8 )  value = new Integer(in.readUnsignedByte() );
    else if(size==16)  value = new Integer(in.readUnsignedShort());
    else if(size==32)  value = in.readUnsignedBits(size);
}

/***************************************
* if that didn't work, read bit-by-bit *
***************************************/
if(value==null) value = in.readUnsignedBits(size);

} // end of read method


/**********************************************************************
* method to write this data field 
**********************************************************************/
public void write(BitOutputStream out) throws IOException {

boolean aligned = out.isByteAligned();

if(aligned && size==8 )  out.writeByte( value.intValue() );
else if(aligned && size==16)  out.writeShort(value.intValue() );
else if(aligned && size==32)  out.writeInt(  value.intValue() );
else if(aligned && size==64)  out.writeLong( value.longValue());
else {
    /*******************
    * write bit-by-bit *
    *******************/
    if(size< 32) out.writeBits(value.intValue() , size);
    else         out.writeBits(value.longValue(), size);

} 


} // end of write method


} // end of UnsignedIntField class
