import java.io.*;

public class BCDH extends DataBlock {


byte[] raw;
int nbytes;


BCDHStructure structure;


/***************************************************************************
* constructor
***************************************************************************/
public BCDH() {

resetRawStorage();

} // end of constructor

/***************************************************************************
* 
***************************************************************************/
public void setStructure(BCDHStructure structure) { 

    this.structure = structure;
    reparse();
}

/***************************************************************************
* 
***************************************************************************/
public BCDHStructure getStructure() { return structure; }

/***************************************************************************
* return the raw data byte containing the ID code
***************************************************************************/
public byte getCodeByte() { return raw[3]; }


/***************************************************************************
* reset raw storage
***************************************************************************/
public void resetRawStorage() {

nbytes=0;
raw = new byte[4];

}


/***************************************************************************
* read a 4 byte "dword" into the raw data array
***************************************************************************/
private void readDWord(BitInputStream in) throws IOException {

/**********************************************************
* allocate 4 more bytes of raw data storage if we need to *
**********************************************************/
if(raw.length < nbytes+4) {
    byte[] new_raw = new byte[nbytes+4];
    for(int i=0; i<nbytes; ++i) {
        new_raw[i] = raw[i];
    }
    raw = new_raw;
}


/****************
* read the data *
****************/
in.mark(4);
in.readFully(raw, nbytes, 4);
nbytes += 4;

} // end of readDWord method

/***************************************************************************
* push the last 4 byte dword back into the input stream. Note this 
* can only be called once, and the argument must be the same input stream
* that the last read was made from.
***************************************************************************/
private void pushBackDWord(BitInputStream in) throws IOException {

    in.reset();
    nbytes -= 4;

} // end of pushBackDWord method

/***************************************************************************
* returns true if the last DWord read was a continuation dword
***************************************************************************/
private boolean readContinuationDWord() {

return (raw[nbytes-1] & 0x0f) == 1;

} // end of readContinuationDWord method



/***************************************************************************
* read the packet
***************************************************************************/
public void read(BitInputStream in) throws IOException {

/*************************************
* empty out any previously read data *
*************************************/
removeAllChildren();
nbytes=0;

/******************************
* read the first 4 byte block *
******************************/
readDWord(in);

/*************************************************************
* now read until we find a dword which is not a continuation
* or we run out of data
*************************************************************/
try {
    do {
        readDWord(in);
    } while(readContinuationDWord() );

    /**********************************************************
    * if we got here we have read 4 bytes too far, so reset
    * the input stream and the raw data array
    *********************************************************/
    pushBackDWord(in);

} catch(EOFException e) {
    /*******************************************
    * we hit the end of the file but that's OK *
    *******************************************/
}

/******************************
* now interpret the raw bytes *
******************************/
RawDataBlock data = new RawDataBlock(raw);
data.resize(nbytes);

add(data);

reparse();


} // end of read method


/*****************************************************************************
* 
*****************************************************************************/
private void reparse() {

/*******************************************************************
* make sure the structure is set and that we have raw data to read *
*******************************************************************/
if(structure==null) return;
if(nbytes==0) return;

String[] formats = structure.getFormats();
//if(formats==null) System.out.println("no format for "+this);
if(formats==null) return;

for(int i=0; i<formats.length; ++i) {

    DataBlock block=null;
    try { 

        block = new DataBlock(formats[i]);
        if(block.size() != 8*nbytes) continue;


        BitInputStream in = new BitInputStream(new ByteArrayInputStream(raw));
        block.read(in);
    } catch(IOException e) { System.out.println(e); return; }

    /***********************************************************
    * get rid of any children and copy the block we just read
    * into this one.
    * Note when we add a child to this node it gets removed from
    * the block, so that's why the index to block.get() is 
    * always zero.
    ***********************************************************/
    removeAllChildren();
    int nchildren=block.getChildCount();
    for(int j=0; j<nchildren; ++j) {

        add(block.get(0));
    }

//System.out.println(get("ph").formattedValue() );
    /*****************************************************
    * if we got here, then we found a format which works *
    *****************************************************/
    return;

}

/*******************************************
* if we got here we couldn't find a format *
*******************************************/
return;



} // end of reparse method


/*****************************************************************************
* string representation used in the tree display
*****************************************************************************/
public String toString() { 
    if(structure==null) return "Generic BCDH Packet";
    else {
        StringBuffer buffer = new StringBuffer();

        buffer.append(structure.getName());

        String[] highlights = structure.getHighlights();
        if(highlights != null) {

            buffer.append(" / ");
            for(int i=0; i<highlights.length; ++i) {

//System.out.println("highlight of "+structure.getName()+" "+i+" "+highlights[i]);

                DataField field = get(highlights[i]);
                if(field != null) {

                    buffer.append(highlights[i]);
                    buffer.append("=");
                    buffer.append(field.formattedValue());
                    buffer.append(" ");
                }

            } // end of loop over highlights

        } // end if there are highlights

        return buffer.toString();

    } // end if there is a strufture defined

} // end of toString method


} // end of BCDH class
