import java.io.*;
import javax.swing.tree.*;
import java.awt.*;
import javax.swing.*;
import java.util.*;

public class BCDHStream extends DataStream {

java.util.List structures;

/******************************************************************************
* constructor
******************************************************************************/
public BCDHStream() {

super(new BCDH() );

try {
    readIDList(FileSearchPath.getClassPath().find("bcdh.list"));
} catch(IOException e) {
    System.out.println(e);
}

} // end of constructor


/****************************************************************************
* 
****************************************************************************/
public void readIDList(File file) throws IOException {

structures = new ArrayList();

/*******************
* read config file *
*******************/
BufferedReader in = new BufferedReader(new FileReader(file));

String line;
while((line= in.readLine()) != null) {

    line = line.trim();
    /********************************
    * skip comments and blank lines *
    ********************************/
    if(line.startsWith("#") || line.length() == 0) continue;

    /*****************
    * parse the line *
    *****************/
    StringTokenizer tokenizer = new StringTokenizer(line, "|");

    String bits    = tokenizer.nextToken().trim();
    String name    = tokenizer.nextToken().trim();

    String format_list = null;
    String highlights_list = null;
    try { 
        format_list    = tokenizer.nextToken().trim(); 
        highlights_list = tokenizer.nextToken().trim();

    } catch(NoSuchElementException e) {}

    /************************
    * parse the bits string *
    ************************/
    byte mask=0;
    byte value=0;
    for(int i=0; i<bits.length(); ++i) {

        char c = bits.charAt(i);

        if(c == '1' ) {
            value <<=1;
            value |= 1;
        
            mask <<=1;
            mask |= 1;
        } else if(c == '0' ) {
            value <<=1;

            mask <<=1;
            mask |= 1;
        }    

    } // end of loop over binary digits

//System.out.println(bits+"|"+name);

    /************************
    * parse the format list *
    ************************/
    String[] formats=null;
    if(format_list != null) {

        tokenizer = new StringTokenizer(format_list,",");
        formats = new String[tokenizer.countTokens()];

        for(int i=0; i<formats.length; ++i) {
            formats[i] = tokenizer.nextToken().trim();
        }
    }

    /******************************
    * parse the highlights string *
    ******************************/
//System.out.println("highlights_list="+highlights_list);
    String[] highlights=null;
    if(highlights_list != null) {

        tokenizer = new StringTokenizer(highlights_list,",");
        highlights = new String[tokenizer.countTokens()];

        for(int i=0; i<highlights.length; ++i) {
            highlights[i] = tokenizer.nextToken().trim();
//System.out.println("parsing "+i+" highlight="+highlights[i]);
        }
    }
    

    /********************************************
    * add a structure for this type to the list *
    ********************************************/
    structures.add(new BCDHStructure(mask,value,name, formats, highlights) );

} // end of loop over lines



} // end of readIDList method
    
/****************************************************************************
* read a single "unit" of data from the stream
****************************************************************************/
protected DataNode readUnit(BitInputStream in) throws IOException {

/********************
* read the raw data *
********************/
BCDH bcdh = new BCDH();
bcdh.read(in);

/********************
* determine the type *
*********************/
for(Iterator it = structures.iterator(); it.hasNext(); ) {
    BCDHStructure structure =(BCDHStructure)it.next();
    if(structure.codeMatches(bcdh.getCodeByte()) ) {
        /****************
        * found a match *
        ****************/
        bcdh.setStructure(structure);
        break;
    }
}

return bcdh;

/***************************
* add the unit to the tree *
***************************/
/*
        BCDHStructure structure = bcdh.getStructure();
        if(structure != null 
           && structure.getFormats() == null) {
        //   &&  structure.getName().equals("BCDH Absolute Timestamp Echo") ) {

      //   &&  !structure.getName().equals("DM Normal Event") ) {
System.out.println(i + " "+bcdh);
            add(bcdh);
        } else --i;

*/


} // end of readUnit method

/***************************************************************************
* 
***************************************************************************/
protected void readHeader(BitInputStream in) throws IOException {
    DataBlock header = new DataBlock("BATStdLDPHead");
    header.read(in);
    add(header);

    int size = header.get("size").value().intValue();
    int leftover = size - header.size()/8;
    if(leftover<0) throw new IOException("Inconsistent header size");

    RawDataBlock raw = new RawDataBlock(0);
    raw.resize(leftover);
    raw.read(in);
    add(raw);


} // end of readHeader method



/***************************************************************************
* main method for testing 
***************************************************************************/
public static void main(String[] args) throws Exception {

BCDHStream stream = new BCDHStream();


/*
BitInputStream in = new BitInputStream(
                    new BufferedInputStream(
                    new FileInputStream("dmc_asic.dat.reverse")));
*/


BitInputStream in = new BitInputStream(
                    new BufferedInputStream(
                    new FileInputStream("/usr/home/pier/swift/telemetry/bat/david/e0023diz.evl")));




stream.read(in);

JFrame frame = new JFrame();
JTree tree = new JTree(stream);

Renderer renderer = new Renderer(tree);

tree.setCellRenderer(renderer);
tree.setCellEditor(renderer);
tree.setEditable(true);

JScrollPane scroll = new JScrollPane(tree);
scroll.setPreferredSize(new Dimension(500,700));
frame.getContentPane().add(scroll, BorderLayout.CENTER);

frame.pack();
frame.setVisible(true);

} // end of main method


} // end of BCDHStream class
