import java.io.*;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.event.*;

public class ExcelFile extends BiffStream implements TableModel{

String[] shared_strings;

int nrows;
int ncols;
Object[][] data;

/*****************************************************************************
*
*****************************************************************************/
public ExcelFile() throws IOException {super(); }


/*****************************************************************************
*
*****************************************************************************/
public Object[][] getData() { return data; }



/*****************************************************************************
*
*****************************************************************************/
public void read(BitInputStream in) throws IOException {

/************************
* read the BIFF records *
************************/
super.read(in);


System.out.println("read stream");

/**********************************
* extract the shared string table *
**********************************/
for(int i=0; i<getChildCount(); ++i) {

    Record rec = (Record)get(i);
//System.out.println(rec);
    if(rec.getType() == Record.SST_TYPE) {
        /*************************
        * found the table record *
        *************************/
//System.out.println("found shared string table");
        SST sst = (SST)rec.getData();
        shared_strings = new String[sst.getStringCount()];
        for(int j=0; j<shared_strings.length; ++j) {

             shared_strings[j] = sst.getString(j);

        }
    }

}

//System.out.println("got shared string table");

/*****************************************
* figure out the dimensions of the table *
*****************************************/
int max_row=-1;
int max_col=-1;
for(int i=0; i<getChildCount(); ++i) {

    Record rec = (Record)get(i);
    if(rec.isCellContents()) {
        DataBlock cell = rec.getData();
        int row = cell.get("row").value().intValue();
        int col = cell.get("col").value().intValue();

        if(row > max_row ) max_row = row;
        if(col > max_col ) max_col = col;
    }
}

nrows = max_row+1;
ncols = max_col+1;

data = new Object[nrows][ncols];

//System.out.println("nrows="+nrows+" ncols="+ncols);

/*******************
* extract the data *
*******************/
for(int i=0; i<getChildCount(); ++i) {


    Record rec = (Record)get(i);

    if(rec.isCellContents() ) {
        int type = rec.getType();

//System.out.println(i+" "+rec);

        DataBlock cell = rec.getData();
        int row = cell.get("row").value().intValue();
        int col = cell.get("col").value().intValue();

//System.out.println(i+" "+rec+" row="+row+" col="+col);

        if(type == Record.SSL_TYPE) {
            /**********************
            * shared string label *
            **********************/
            int index = cell.get("sst_index").value().intValue();

            if(shared_strings!=null && index < shared_strings.length) {
                data[row][col] = shared_strings[index];
            } else {
                data[row][col]= "string index "+index;
            }
    
        } else if(type == Record.RK_TYPE) {
            /******************
            * RK number value *
            ******************/
            data[row][col] = cell.get("value").value();
        }

//System.out.println(row+" "+col+" "+data[row][col]);
    } // end if is cell contents
}


} // end of read method

/*****************************************************************************
* print the contents of the table to a given Writer using the given 
* field separator
*****************************************************************************/
public void print(Writer w, String separator) throws IOException {

for(int i=0; i<nrows; ++i) {
    for(int j=0; j<ncols; ++j ) {

        if(data[i][j] != null) w.write(data[i][j].toString());
        if(j!= ncols-1) w.write(separator);
    }
    w.write("\n");
}

} // end of print method


/*****************************************************************************
* The following methods implement the TableModel interface
*****************************************************************************/
public void addTableModelListener(TableModelListener l) {}
public void removeTableModelListener(TableModelListener l) {}

public Class getColumnClass(int columnIndex) {return Object.class; }

public int getColumnCount() {return ncols; }
public int getRowCount()    {return nrows;}


public String getColumnName(int columnIndex) {return "col"+columnIndex; }




public Object getValueAt(int rowIndex, int columnIndex) {
    return data[rowIndex][columnIndex];
}

public boolean isCellEditable(int rowIndex, int columnIndex) {return false; }


public void setValueAt(Object aValue, int rowIndex, int columnIndex) {}
        


/*****************************************************************************
* for testing
*****************************************************************************/
public static void main(String[] args) throws IOException {

BitInputStream in = new BitInputStream(
                    new BufferedInputStream(
                    new FileInputStream("XRT-PSU-028X2.fd") ));

ExcelFile file = new ExcelFile();
file.read(in);

Writer w = new OutputStreamWriter(System.out);
file.print(w, "|");
w.close();


JFrame frame = new JFrame();
JTable table = new JTable(file);
frame.getContentPane().add(table);

frame.pack();
frame.setVisible(true);

} // end of main method

} // end of ExcelFile class
