import java.io.*;

public class Record extends DataBlock {

public static final int SST_TYPE = 252; // shared string table
public static final int SSL_TYPE = 253; // shared string label
public static final int  RK_TYPE = 638; // RK value 

String description;



/*****************************************************************************
*
*****************************************************************************/
public Record() throws IOException { super("RecordHeader"); } 

/*****************************************************************************
*
*****************************************************************************/
public int getType() { return get("code").value().intValue(); }

/*****************************************************************************
* return true if this record describes the contents of a cell
*****************************************************************************/
public boolean isCellContents() { 

int type = getType();
return type == SSL_TYPE || type == RK_TYPE;

} // end of isCellContents

/*****************************************************************************
* return the DataBlock containing the data for this record
*****************************************************************************/
public DataBlock getData() { return (DataBlock)get(getChildCount()-1); }

/*****************************************************************************
*
*****************************************************************************/
public void read(BitInputStream in) throws IOException {

//System.out.println("-------about to read record "+in.available()+" bytes available");


/******************
* read the header *
******************/
super.read(in);


int type = get("code").value().intValue();
int length = get("length").value().intValue();

//System.out.println("-------type="+type+" length="+length);

DataBlock block=null;

if(type == 125) {
    /**************
    * Column Info *
    **************/
    description = "Column Info";
    block = new DataBlock("ColInfo");

} else if(type == 190) {
    /************************
    * Multiple Blank record *
    ************************/
    description = "Multiple Blanks";
    block = new MulBlank(length);

} else if(type == 224) {
    /*************************
    * Extended Format record *
    *************************/
    description = "Extended Format Record";
    block = new DataBlock("XFRecord");

} else if(type == 252) {
    /**********************
    * Shared String Table *
    **********************/
    description = "Shared String Table";
    block = new SST();

} else if(type == 253) {
    /**********************
    * Shared String Label *
    **********************/
    description = "Shared String Label";
    block = new DataBlock("LabelSST");

} else if(type == 638) {
    /***********
    * RK value *
    ***********/
    description = "RK value";
    block = new DataBlock("RK");

} else if(type == 1054) {
    /*************************
    * Extended Format record *
    *************************/
    description = "Format Record";
    block = new DataBlock("Format");



} else {
    /***************
    * unknown type *
    ***************/
    description="length="+get("length").formattedValue();

//System.out.println("-------about to read "+length+" bytes");

    if(length>0) {
        RawDataBlock raw = new RawDataBlock(length);
        //TextBlock raw = new TextBlock(length);
        raw.read(in);
        add(raw);
    }

//System.out.println("-------got it");
}

if(block!=null) {
    if(block.size()/8>length) {
        System.out.println("block size "+(block.size()/8)+
                           " is greater than length "+length);
    }
    block.read(in);

    if(block.size()/8 != length) {
        System.out.println("block size "+(block.size()/8)+
                           " is different from length "+length);
    }

    add(block);

} // end if we have a custom block to read


} // end of read method

/*****************************************************************************
*
*****************************************************************************/
public String toString() {

    return  "Record "+get("code"  ).formattedValue()+
            " "+description;

} // end of toString method

} // end of Record class
