import java.io.*;
import java.util.*;

public class APIDKnower {

/*******
* data *
*******/
Map map;

/***************************************************************************
* constructor 
***************************************************************************/
public APIDKnower() {

map = new HashMap();

} // end of APIDKnower

/***************************************************************************
* constructor 
***************************************************************************/
public APIDKnower(File file) throws IOException {

this();

//System.out.println("creating APIDKnower from "+file);

/*******************
* read config file *
*******************/
BufferedReader in = new BufferedReader(new FileReader(file));

String line;
while((line= in.readLine()) != null) {

    line = line.trim();
    /********************************
    * skip comments and blank lines *
    ********************************/
    if(line.startsWith("#") || line.length() == 0) continue;

    /*****************
    * parse the line *
    *****************/
    StringTokenizer tokenizer = new NullTokenizer(line,"|");

    int apid = Integer.parseInt(tokenizer.nextToken().trim());



    /******************************
    * create the packet structure *
    ******************************/
    PacketStructure structure = new PacketStructure();

    /************************
    * parse the header list *
    ************************/
    String headers  = tokenizer.nextToken().trim();
    for(StringTokenizer t = new StringTokenizer(headers); t.hasMoreTokens(); ) {

        structure.addHeader(t.nextToken() );

    }

    /*************************
    * parse the trailer list *
    *************************/
    String trailers = tokenizer.nextToken().trim();
    for(StringTokenizer t=new StringTokenizer(trailers); t.hasMoreTokens(); ) {

        structure.addTrailer(t.nextToken() );
    }

    /***************************
    * parse the highlight list *
    ***************************/
    String highlights = tokenizer.nextToken().trim();
    for(StringTokenizer t=new StringTokenizer(highlights); 
        t.hasMoreTokens(); ) {

        String token = t.nextToken();
        int dot = token.indexOf('.');

        int header=0;
        if(dot != -1) header = Integer.parseInt(token.substring(0,dot) );

        String name = token.substring(dot+1);
        structure.addHighlight(header, name);
    }

    
    /****************************
    * get the packet description *
    *****************************/
    structure.setDescription(tokenizer.nextToken() );    

    /*************************
    * remember the structure *
    *************************/
    add(apid,structure);

} // end of loop over lines


} // end of constructor


/***************************************************************************
* add an apid - PacketStructure pair 
***************************************************************************/
public void add(int apid, PacketStructure structure) {

    map.put(new Integer(apid), structure);
}

/***************************************************************************
* return the PacketStructure for a given APID
* if no structure is known for that APID, returns a new empty structure
***************************************************************************/
public PacketStructure getStructure(int apid) {

PacketStructure structure = (PacketStructure)map.get(new Integer(apid));
if(structure == null) {
    /**************************************************
    * unknown packet type, create a default structure *
    **************************************************/
    structure = new PacketStructure();
    structure.addHighlight(0,"apid");
    structure.setDescription("Unknown Packet Type");
}

return structure;

} // end of getStructure method

/***************************************************************************
* return a list of APIDs and their descriptions 
***************************************************************************/
public List list() {

SortedSet keys = new TreeSet(map.keySet() );

List list = new ArrayList(keys.size());
for(Iterator it = keys.iterator(); it.hasNext(); ) {

    Integer key = (Integer)it.next();
    PacketStructure value = (PacketStructure)map.get(key);

    list.add(key + " "+ value.getDescription() );

}

return list;

} // end of list method

/***************************************************************************
* read a packet from an input bit stream, building its structure according
* to its APID 
***************************************************************************/
public Packet readPacket(BitInputStream in) throws IOException {

/**********************************************
* create a packet and read its primary header *
**********************************************/
Packet p = new Packet();
p.setStructureIsMutable(false);
p.readPrimaryHeader(in);

/****************************************************************
* get the PAID from the primary header and set up the structure *
****************************************************************/
int apid = p.getHeader(0).get("apid").value().intValue();
PacketStructure structure = getStructure(apid);
structure.install(p);
        
/******************************
* read the read of the packet *
******************************/
p.readBody(in);

return p;


} // end of readPacket


/***************************************************************************
* make a new packet with a given APID
***************************************************************************/
public Packet makePacket(int apid) throws IOException {

/******************
* create a packet *
******************/
Packet p = new Packet();
p.setStructureIsMutable(false);

/*************************************
* set the APID in the primary header *
*************************************/
p.getHeader(0).get("apid").value(new Integer(apid));

/***********************
* set up its structure *
***********************/
PacketStructure structure = getStructure(apid);
structure.install(p);

p.setLength();
        
return p;

} // end of makePacket method

} // end of APIDKnower class

