import java.io.*;
import javax.swing.tree.*;
import java.awt.*;
import javax.swing.*;
import java.util.*;


public class Packet extends DataBlock {


/****************************************************************************
* embedded class used to create a string representation of one header value
****************************************************************************/
private class Highlight {

    /*******
    * data *
    *******/
    private int header;
    private String name;

    /**************
    * constructor *
    **************/
    public Highlight(int header, String name) {
        this.header=header;
        this.name=name;
    } // end of constructor

    /************************
    * string representation *
    ************************/
    public String toString() {
        DataBlock head = getHeader(header);
        return name+"="+head.get(name).formattedValue();
    } // end of toString method

} // end of Highlight embedded class
    


/*******
* data *
*******/

private static final int CHUNK=1024;

private DataBlock primary;
private DataBlock headers;
//private DataNode raw;
private DataBlock trailers;
private java.util.List highlights;
private String description;

/***************************************************************************
* constructor 
***************************************************************************/
public Packet() throws IOException {

this(new DataBlock("CCSDSHead") );

/****************************
* set default header values *
****************************/
primary.get("isFirst").value(new Integer(1));
primary.get("isLast" ).value(new Integer(1));
primary.get("hasHead2").value(new Integer(1));


} // end of constructor

/***************************************************************************
* constructor using an existing primary header
***************************************************************************/
public Packet(DataBlock primary) throws IOException {

headers = new DataBlock();
trailers = null;

headers.name("Packet header(s)");

this.primary = primary;
headers.add(primary);

super.add(headers);
super.add(new RawDataBlock(0)); // just a place holder

highlights=new ArrayList();

addHighlight(0,"apid");
addHighlight(0,"sequence");

description = new String();

} // end of constructor

/***************************************************************************
* accessors for the packet description 
***************************************************************************/
public   void setDescription(String descr) {        description = descr; }
public String getDescription(            ) { return description; }

/***************************************************************************
* set the packet length field to the correct value for the size of the packet 
***************************************************************************/
public void setLength() {

System.out.println("size="+size());

    primary.get("length").value(new Integer((size()+7)/8-1-6));
}



/***************************************************************************
* methods for adding header and trailer data blocks 
***************************************************************************/
public void addHeader(DataNode node) { headers.add(node); }

public void add(DataNode node) { addHeader(node); }

public DataBlock getHeader(int index) { return (DataBlock)headers.get(index); }
public DataBlock getTrailer(int index) { return (DataBlock)trailers.get(index);}


public void addTrailer(DataNode node) { 

if(trailers == null ) {
    /****************************
    * this is the first trailer *
    ****************************/
    trailers = new DataBlock();
    trailers.name("Packet trailer(s)");
    super.add(trailers);
}

trailers.add(node); 

} // end of addTrailer method

/***************************************************************************
* add a highlight
***************************************************************************/
public void addHighlight(int header, String name) {

    highlights.add(new Highlight(header,name));
}

/***************************************************************************
* remove all the highlights
***************************************************************************/
public void clearHighlights() { highlights.clear(); }

/***************************************************************************
* string representation
***************************************************************************/
public String toString() {

StringBuffer string = new StringBuffer();

for(Iterator it = highlights.iterator(); it.hasNext(); ) {
    Highlight highlight = (Highlight)it.next();

    string.append(highlight.toString() );
    if(it.hasNext()) string.append(" ");

}

string.append(" / "+description);

return string.toString();

} // end of toString method

/***************************************************************************
* read the raw data field
***************************************************************************/
private void readRawData(BitInputStream in) throws IOException {

    /*********************************
    * determine the size of the data *
    *********************************/
    int full_size = primary.get("length").value().intValue() +7;
    int data_size =      full_size -  (headers.size()+7)/8 ;
    if(trailers != null) data_size -= (trailers.size()+7)/8;

    /********************************************
    * determione how we will represent the data *
    ********************************************/
    DataNode raw;
    if(data_size>CHUNK) raw = new SplitRawDataBlock(data_size, CHUNK);
    else                raw = new      RawDataBlock(data_size);

    /**********************************************************
    * replace the existing data representation with a new one *
    **********************************************************/
    remove(1);
    insert(raw,1);

    /****************
    * read the data *
    ****************/
    raw.read(in);

} // end of readRawData method

/***************************************************************************
* read the packet
***************************************************************************/
public void read(BitInputStream in) throws IOException {

/*******************
* read the headers *
*******************/
headers.read(in);

/**************************************
* set the data size and read the data *
**************************************/
readRawData(in);

/*************************************
* read the trailers if there are any *
*************************************/
if(trailers != null) trailers.read(in);


} // end of read method

/***************************************************************************
* read only the primary header of the packet.
***************************************************************************/
public void readPrimaryHeader(BitInputStream in) throws IOException {

primary.read(in);

//System.out.println(   "apid="+primary.get("apid")+
//                   " length="+primary.get("length"));

} // end of readPrimaryHeader method

/***************************************************************************
* read every thing after the primary header of the packet
* presumably the primary header has been read by some other means.
***************************************************************************/
public void readBody(BitInputStream in) throws IOException {

/*******************
* read the headers *
*******************/
for(int i=1; i<headers.getChildCount(); ++i) {
    headers.get(i).read(in);
}

/**************************************
* set the data size and read the data *
**************************************/
readRawData(in);

/*************************************
* read the trailers if there are any *
*************************************/
if(trailers != null) trailers.read(in);


} // end of read method



/***************************************************************************
* main method for testing 
***************************************************************************/
public static void main(String[] args) throws Exception {

Packet p = new Packet();
p.addHeader(new DataBlock("SwiftHead2") );

BitInputStream in = new BitInputStream(new FileInputStream("zwift_0_0_0.ldp"));
p.read(in);

JFrame frame = new JFrame();
JTree tree = new JTree(p);
JScrollPane scrollPane = new JScrollPane(tree);
frame.getContentPane().add(scrollPane, BorderLayout.CENTER);

frame.pack();
frame.setVisible(true);

} // end of main method


} // end of Packet class

