import java.io.*;
import javax.swing.tree.*;
import java.awt.*;
import javax.swing.*;


public class PacketStream extends DataStream {

/*******
* data *
*******/
private APIDKnower apids;



/****************************************************************************
* constructor
****************************************************************************/
PacketStream()  throws IOException {
super(new Packet());

/***************************
* find the APID dictionary *
***************************/
try {
    apids = new APIDKnower(FileSearchPath.getClassPath().find("apid.list") );
} catch(Exception e) {
    apids = new APIDKnower();
}

} // end of constructor



/****************************************************************************
* read a single "unit" of data from the stream
****************************************************************************/
protected DataNode readUnit(BitInputStream in) throws IOException {

Packet p = new Packet();
p.readPrimaryHeader(in);
int apid = p.getHeader(0).get("apid").value().intValue();
PacketStructure structure = apids.getStructure(apid);
structure.install(p);
p.readBody(in);

p.setStructureIsMutable(false);

return p;

} // end of readUnit method



/***************************************************************************
* main method for testing 
***************************************************************************/
public String toString() { return "Packet Stream"; }


/***************************************************************************
* main method for testing 
***************************************************************************/
public static void main(String[] args) throws Exception {

APIDKnower apids = new APIDKnower(new File("apid.list"));

PacketStream packets = new PacketStream();

BitInputStream in = new BitInputStream(new FileInputStream("doc.ccsds"));
packets.read(in);

JFrame frame = new JFrame();
JTree tree = new JTree(packets);

Renderer renderer = new Renderer(tree);

tree.setCellRenderer(renderer);
tree.setCellEditor(renderer);
tree.setEditable(true);

JScrollPane scrollPane = new JScrollPane(tree);
frame.getContentPane().add(scrollPane, BorderLayout.CENTER);

frame.pack();
frame.setVisible(true);

} // end of main method

} // end of PacketStream class

