import java.io.*;

public class Constant extends DataBlock {

int index;

/****************************************************************************
*
****************************************************************************/
public Constant(int index) { this.index = index; }


/****************************************************************************
*
****************************************************************************/
public void read(BitInputStream in) throws IOException {

AbstractDataField tag = new UnsignedIntField("tag", 8, "Data type code");
tag.read(in);
add(tag);

int code = tag.value().intValue();
if(code==1) {
    /***************
    * UTF-8 string *
    ***************/
    AbstractDataField field = new UnsignedIntField("length", 16, 
                                                   "length in bytes");
    field.read(in);
    add(field);
    
    int length = field.value().intValue();
    field = new ASCIIDataField("value", 8*length, "Value");
    field.read(in);
    add(field);

} else if(code==3) {
    /**********
    * integer *
    **********/
    AbstractDataField field = new SignedIntField("value", 32, "Value");
    field.read(in);
    add(field);

} else if(code==4) {
    /********
    * float *
    ********/
    AbstractDataField field = new FloatField("value", "Value");
    field.read(in);
    add(field);

} else if(code==5) {
    /********
    * long *
    ********/
    AbstractDataField field = new SignedIntField("value", 64, "Value");
    field.read(in);
    add(field);

} else if(code==6) {
    /********
    * double *
    ********/
    AbstractDataField field = new DoubleField("value", "Value");
    field.read(in);
    add(field);

} else if(code==7) {
    /********
    * class *
    ********/
    AbstractDataField field = new UnsignedIntField("index", 16, 
                                                   "Index of class name");
    field.read(in);
    add(field);

} else if(code==8) {
    /********
    * string *
    ********/
    AbstractDataField field = new UnsignedIntField("index", 16, 
                                                   "Index of string contents");
    field.read(in);
    add(field);

} else if(code==9 || code==10 || code==11) {
    /***************************************
    * field, method or interface reference *
    ***************************************/
    AbstractDataField field = new UnsignedIntField("class", 16, 
                                                   "Index of class");
    field.read(in);
    add(field);

    field = new UnsignedIntField("name_type", 16, "Index of Name and Type");
    field.read(in);
    add(field);

} else if(code==12) {
    /*********************
    * name and type info *
    *********************/
    AbstractDataField field = new UnsignedIntField("name", 16, 
                                                   "Index of field name");
    field.read(in);
    add(field);

    field = new UnsignedIntField("type", 16, "Index of field descriptor");
    field.read(in);
    add(field);
} else {
    /**********
    * unknown *
    **********/
  //  throw new IOException("Unknown constant type "+code);
}




} // end of read method

/****************************************************************************
*
****************************************************************************/
public String toString() { 

try { 
    return "Constant #"+index+": "+
           ((DataField)get(getChildCount()-1)).formattedValue(); 

} catch(Exception e) { return "Constant"; }

}

} // end of Constant class
