import java.io.*;
public class ID3Frame extends DataBlock {

String contents;
int version;
int max_length;

/*****************************************************************************
*
*****************************************************************************/
public ID3Frame(int version, int max_length) { 
    this.version=version;
    this.max_length=max_length;
}



/*****************************************************************************
*
*****************************************************************************/
public void read(BitInputStream in ) throws IOException {

DataBlock head;

/***************************************
* version two has only a 6 byte header *
***************************************/
if(version<=2) head = new DataBlock("ID3FrameHeadv2");
else           head = new DataBlock("ID3FrameHead");

head.read(in);
add(head);


/***********************************************************
* calculate the tag length
* It's synchsafed in versions 4 and higher
***********************************************************/
int length;
if(version>=4) length = (int)ID3Tag.decodeSynchSafeField(head.get("length"));
else           length = head.get("length").value().intValue();

if(length==0 || length>max_length) length=max_length-head.size()/8;

System.out.println("frame length = "+length);
//if(length > 1125 ) length=0;


if(length<=80) {
    ASCIIDataField field = new ASCIIDataField("Text",length*8, "text");
    field.read(in);
    contents = field.formattedValue();
    add(field);
} else {
    TextBlock raw = new TextBlock(length);
    raw.read(in);
    add(raw);
    contents = "("+length+" charaters)";
}



} // end of read method

/*****************************************************************************
*
*****************************************************************************/
public String toString() { 

    return "ID3 Frame " + 
           ((DataBlock)get(0)).get("id").formattedValue() +
           " = "+contents; 

}

} // end of ID3Frame class

