import java.io.*;

public class ID3Tag extends DataBlock {

/*****************************************************************************
*
*****************************************************************************/
public void read(BitInputStream in ) throws IOException {


DataBlock head = new DataBlock("ID3TagHeader");
in.mark(head.size()/8);
head.read(in);
add(head);

if(!head.get("id").formattedValue().equals("ID3") ) {
    in.reset();
    throw new IOException("Not an ID3 Tag");
}

int version = head.get("version").value().intValue();

/****************************
* get the length of the tag *
****************************/
int length = (int)decodeSynchSafeField(head.get("length"));

//RawDataBlock raw = new RawDataBlock(length);
//raw.read(in);
//add(raw);

/**********************
* read all the frames *
**********************/
while(length>0) {

    System.out.println("length="+length);

    ID3Frame frame = new ID3Frame(version, length);
    frame.read(in);
    add(frame);

    length -= frame.size()/8;
}



if(head.get("footer").value().intValue() == 1) {
    /****************
    * read a footer *
    ****************/
    RawDataBlock foot = new RawDataBlock(10);
    foot.read(in);
    add(foot);
}




} // end of read method


/*****************************************************************************
* IP3 tags store integers using only the lowest 7 bits of each byte.
* so that they cannot be mistaken for MP3 frame synch codes
*****************************************************************************/
public static long decodeSynchSafeField(DataField field) {

    long raw = field.value().longValue();
    int nbytes = field.size()/8;

    long value =0;
    for(int i=nbytes-1; i>=0; --i) {

        value <<= 7;
        value |= raw>>(8*i) & 0x7f;

    }

    return value;


} // end of decodeSynchSafeField method
        
        
/*****************************************************************************
*
*****************************************************************************/
public String toString() { return "ID3 Tag"; }


} // end of ID3Tag class
