import java.io.*;

public class MP3Frame extends DataBlock {

private static final int bit_rate[][][] = {
                                     { {  0,   0,   0,}, {  0,   0,   0,}},
                                     { { 32,  32,  32,}, { 32,  32,   8,}},
                                     { { 64,  48,  40,}, { 64,  48,  16,}},
                                     { { 96,  56,  48,}, { 96,  56,  24,}},
                                     { {128,  64,  56,}, {128,  64,  32,}},
                                     { {160,  80,  64,}, {160,  80,  64,}},
                                     { {192,  96,  80,}, {192,  96,  80,}},
                                     { {224, 112,  96,}, {224, 112,  56,}},
                                     { {256, 128, 112,}, {256, 128,  64,}},
                                     { {288, 160, 128,}, {288, 160, 128,}},
                                     { {320, 192, 160,}, {320, 192, 160,}},
                                     { {352, 224, 192,}, {352, 224, 112,}},
                                     { {384, 256, 224,}, {384, 256, 128,}},
                                     { {416, 320, 256,}, {416, 320, 256,}},
                                     { {448, 384, 320,}, {448, 384, 320,}},
                                     { { -1,  -1,  -1,}, { -1,  -1,  -1,}}  };
                   

private static final int sampling[][] ={ {44100, 22050, 11025},      
                                         {48000, 24000, 12000},
                                         {32000, 16000,  8000}, 
                                         {   -1,    -1,    -1}  };

private static final int sample_size[][] = {{48000, 144000, 144000},
                                            {24000,  72000,  72000} };

/*****************************************************************************
* return the actual decoded bit rate
* returns -1 if it can't be determined or has an illegal value
* returns 0 if the bit rate is free
*****************************************************************************/
int getBitRate() {

try { 
    DataBlock head = (DataBlock)get(0);

    int version=1;
    if(head.get("version").value().intValue() ==3) version=0;
 
    int layer   = 3 - head.get("layer").value().intValue();
    if(layer==3) return -1;

    int code = head.get("bit_rate").value().intValue();

//System.out.println("code="+code+" version="+version+" layer="+layer);

    return bit_rate[code][version][layer];
    
} catch(NullPointerException e) {
    return -1;
}
    

} // end of getBitRate method

/*****************************************************************************
*
*****************************************************************************/
public int getSamplingRate() {

try { 
    DataBlock head = (DataBlock)get(0);

    int version_code = head.get("version").value().intValue();

    int version_index=3;
    if(version_code == 0)      version_index=2;
    else if(version_code == 0) version_index=1;
    else if(version_code == 3) version_index=0;

    int rate_code = head.get("sampling").value().intValue();
    return sampling[rate_code][version_index];

} catch(NullPointerException e) { 
    return -1;
}catch(ArrayIndexOutOfBoundsException e) { 
    return -1;
}

    
} // end of getSamplingRate method

/*****************************************************************************
*
*****************************************************************************/
public int getSampleSize() {

try { 
    DataBlock head = (DataBlock)get(0);

    int version=1;
    if(head.get("version").value().intValue() ==3) version=0;
 
    int layer   = 3 - head.get("layer").value().intValue();
    if(layer==3) return -1;

//System.out.println("code="+code+" version="+version+" layer="+layer);

    return sample_size[version][layer];
    
} catch(NullPointerException e) {
    return -1;
}


} // end of  getSampleSize method


/*****************************************************************************
*
*****************************************************************************/
public void read(BitInputStream in ) throws IOException {

removeAllChildren();

in.mark(4);
DataBlock head = new DataBlock("MP3FrameHead");
head.read(in);
add(head);

if(head.get("synch").value().intValue() != 2047) {
    in.reset();
    throw new IOException("Not a Frame");
}


if(head.get("crc").value().intValue() == 0 ) {
    /*************************
    * frame has a 16 bit CRC *
    *************************/
    UnsignedIntField crc = new UnsignedIntField("CRC", 16, "CRC of this frame");
    crc.read(in);
    add(crc);
}

int padding = head.get("padding").value().intValue();
int length = (getSampleSize()*getBitRate())/getSamplingRate() + padding;

length -= 4;

//System.out.println("bit rate = "+getBitRate() );
//System.out.println("sam rate = "+getSamplingRate() );

if(length <=0 ) return;

in.mark(length+1);
RawDataBlock raw = new RawDataBlock(length);
raw.read(in);
add(raw);

/************************
* peek at the next byte *
************************/
byte next_byte = 0;
try { next_byte = in.readByte(); } 
catch(EOFException e) {}

/*******************************************************
* check to see if we have run on into an ending ID tag *
*******************************************************/
if((raw.size()/8 < length || next_byte != (byte)255 ) && in.available()<128 ) {
    /***************************************************************
    * look back in the raw data to find the pattern "TAG" in ASCII *
    ***************************************************************/
    byte[] array = raw.getByteArray();

    int offset=array.length-1;
    for(int matches=0; matches<3 && offset>=0; --offset) {

        if( (matches==0 && array[offset] == (byte)71) ||
            (matches==1 && array[offset] == (byte)65) ||
            (matches==2 && array[offset] == (byte)84)   ) {
            /***********************
            * found the next match *
            ***********************/
            ++matches;
        } else {
            /*************
            * start over *
            *************/
            matches=0;
        }
    } // end of loop back into array

    /**************************************************
    * now we know how big the array should have been,
    * so clip the end off the raw data block
    * and reset the input stream to just after the end
    * of the clipped data block
    ***************************************************/
    int new_length = offset+1;
    if(new_length>0) {
        raw.resize(new_length);
        in.reset();
        in.skip(new_length);
    } else {
        in.reset();
        in.skip(length);

    }

} else {
    /*****************************************************************
    * everything's fine, but we still have to push back
    * that test byte we read This is awkward since we needed
    * to set the mark back before the raw data were read in case
    * we had gone too far.
    ****************************************************************/
    in.reset();
    in.skip(length);
}


} // end of read method 

/*****************************************************************************
*
*****************************************************************************/
public String toString() { return "MP3 Frame"; }


/****************************************************************************
* read up to the next start code
****************************************************************************/
private RawDataBlock upToNextFrame(BitInputStream in) throws IOException {

    /**********************************************************
    * unknown type read binary data up to the next start code *
    **********************************************************/
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    for(int good_bytes=0; good_bytes<2; ) {
        
        if(good_bytes==0) in.mark(2);
        byte b = in.readByte();
        out.write(b);
        if( (good_bytes==0 && b == (byte)255) ||
            (good_bytes==1 && (b >> 5) == 7)  ) {
            /******************
            * this looks good *
            ******************/
            ++good_bytes;
        } else {
            /***********
            * no match *
            ***********/
            good_bytes=0;
        }
    }

    /***************************
    * push back the start code *
    ***************************/
    in.reset();

    byte[] array = out.toByteArray();
    RawDataBlock raw = new RawDataBlock(array);
    raw.resize(array.length-2);
    return raw;

} // end of upToNextFrame method


} // end of read method
