import java.io.*;

public class MPEGPacket extends DataBlock {

public static final int PICTURE =0;
public static final int SYSTEM_HEADER =187;

public static final int I_TYPE=1;
public static final int P_TYPE=1;
public static final int B_TYPE=1;
public static final int UNKNOWN_TYPE=-1;


String description;

/****************************************************************************
* convenicnce accessor for reading the start code value
* returns -1 if the start code is undefined (e.g. if the packet has
* not been read yet
****************************************************************************/
public int getStartCode() {


    try { return ((DataBlock)get(0)).get("type").value().intValue(); }
    catch(NullPointerException e) { return -1; }

} // end of getStartCode method

/****************************************************************************
* returns true if this is a video slice packet
****************************************************************************/
public boolean isVideoSlice() {
    int code = getStartCode();
    return code>=1 && code <=175;
}


/****************************************************************************
*
****************************************************************************/
public void read(BitInputStream in) throws IOException {

    removeAllChildren();

    /**********************
    * read the start code *
    **********************/
    DataBlock start = new DataBlock("StartCode");
    start.read(in);
    add(start);

/*
    if(start.get("start").value().intValue() != 1) {
        throw new IOException("Invalid start code "
                              +start.get("start").formattedValue() );
    }
*/

    int type = start.get("type").value().intValue();

    if(type==0) {
        /***********************
        * video picture header *
        ***********************/
        description="Video Picture";
        DataBlock head = new DataBlock("Picture");
        head.read(in);
        add(head);

        int frame_type = head.get("type").value().intValue();
        if(frame_type==1) description = description + " (I)";
        if(frame_type==2) description = description + " (P)";
        if(frame_type==3) description = description + " (B)";

//System.out.println("frame type ="+frame_type);
        if(frame_type == 2 || frame_type == 3) {
            /***************
            * P or B frame *
            ***************/
            DataBlock forward = new DataBlock("Vector");
            forward.read(in);
            add(forward);
        }

        if(frame_type==3) {
            /**********
            * B Frame *
            **********/
            DataBlock back = new DataBlock("Vector");
            back.read(in);
            add(back);
        }

        /*****************
        * extra bit info *
        *****************/
        ExtraBitInfo extra = new ExtraBitInfo();
        extra.read(in);
        add(extra);

//System.out.println("extra bytes "+(extra.size()/8) );

        int leftover = in.getBitsToAlign();
        UnsignedIntField align = new UnsignedIntField("align", leftover,
                                                    leftover+" padding bits");
        align.read(in);
        add(align);

    //    RawDataBlock raw = upToNextStartCode(in);
    //    if(raw.size()>0) add(raw);

    } else if(type>=1 && type <=175) {
        /**************
        * video slice *
        **************/
        description="Video Slice";

        DataBlock head = new DataBlock("Slice");
        head.read(in);
        add(head);
        
        ExtraBitInfo extra = new ExtraBitInfo();
        extra.read(in);
        add(extra);

        /******************
        * decode the data *
        ******************/
    //    MacroBlock mb = new MacroBlock();
    //    mb.read(in);


        /***********************
        * read everything left *
        ***********************/
        int leftover = in.getBitsToAlign();
        UnsignedIntField align = new UnsignedIntField("align", leftover,
                                                    leftover+" padding bits");
        align.read(in);
        add(align);
        RawDataBlock raw = upToNextStartCode(in);
        add(raw);



    } else if(type==178) {
        /************
        * user data *
        ************/
        description = "User Data";
        RawDataBlock raw = upToNextStartCode(in);
        add(raw);

    } else  if(type==179) {
        /****************************
        * video sequence start code *
        ****************************/
        description="Video Sequence Start";

        DataBlock head = new DataBlock("VideoHead");
        head.read(in);
        add(head);

        /*********************
        * intra quant matrix *
        *********************/
        UnsignedIntField intra = new UnsignedIntField("intra",1,
                                                    "intra quant matrix flag");
        intra.read(in);
        add(intra);

        if(intra.value().intValue() == 1 ) {
            DataBlock matrix = new DataBlock("IntraQuant");
            matrix.read(in);
            add(matrix);
        }

        /*************************
        * non-intra quant matrix *
        *************************/
        UnsignedIntField non = new UnsignedIntField("intra",1,
                                                    "intra quant matrix flag");
        non.read(in);
        add(non);

        if(non.value().intValue() == 1 ) {
            DataBlock matrix = new DataBlock("IntraQuant");
            matrix.read(in);
            add(matrix);
        }


        RawDataBlock raw = upToNextStartCode(in);
        if(raw.size()>0) add(raw);

    } else if(type==181) {
        /*****************
        * extension data *
        *****************/
        description="Extension Data";
        RawDataBlock raw = upToNextStartCode(in);
        add(raw);

    } else if(type==183) {
        /*****************
        * extension data *
        *****************/
        description="Video Sequence End";
 //       RawDataBlock raw = upToNextStartCode(in);
//        add(raw);




    } else if(type==184) {
        /************
        * GOP start *
        ************/
        description = "Group Of Pictures Start";
        DataBlock gop = new DataBlock("GOP");
        gop.read(in);
        add(gop);

    } else if(type==185) {
        /******************
        * System End code *
        ******************/
        description="System Stream End Code";

    } else if(type == 186) {
        /*******
        * Pack *
        *******/
        DataBlock pack = new DataBlock("PackHeader");
        pack.read(in);
        add(pack);
        description="Pack";
    } else if(type ==187) {
        /*********
        * system *
        *********/
        description="System Header";
        DataBlock system = new DataBlock("SystemHeader");
        system.read(in);
        add(system);

        int length=system.get("length").value().intValue() -6;
        while(true) {
            DataBlock id = new DataBlock("StreamID");
            int size = id.size()/8;
            if(length>=size) {
                id.read(in);
                add(id);
                length-= size;
            } else {
                break;
            }
        }

        if(length>0) throw new IOException("Incorrect System packet length");

    } else if(type == 190 ) {
        /**********
        * padding *
        **********/
        description="Padding";
        UnsignedIntField field = new UnsignedIntField("length",16,"Packet Length");
        field.read(in);
        add(field);
        int length = field.value().intValue();

        RawDataBlock raw = new RawDataBlock(length);
        raw.read(in);
        add(raw);

        

    } else if(type>190) {
        /*********
        * packet *
        *********/
        description="Stream";
        UnsignedIntField field = new UnsignedIntField("length",16,"Packet Length");
        field.read(in);
        add(field);
        int length = field.value().intValue();

        /*********************************
        * read up to 16 stuffing bytes
        * these should be all 0xFF
        *********************************/
        byte[] array = new byte[16];
        int nstuffing=0;
        for(int i=0; i<16; ++i) {
            in.mark(1);
            array[i] = in.readByte();
            if(array[i] != (byte)255 ) {
                in.reset();
                nstuffing=i;
                if(nstuffing>0) {
                    RawDataBlock stuffing = new RawDataBlock(array);
                    stuffing.resize(nstuffing);
                    add(stuffing);
                    length -= nstuffing;
                }
                break;
            }
        }

        /**************
        * buffer size *
        **************/
        in.mark(2);
        DataBlock buffer = new DataBlock("BufferSize");
        buffer.read(in);
        int code = buffer.get("code").value().intValue();
        if(code == 1) {
            /**********
            * keep it *
            **********/
            add(buffer);
            length -= buffer.size()/8;
        } else {
            /**************
            * put it back *
            **************/
            in.reset();
        }
 

//System.out.println("type="+type+" nstuffing="+nstuffing+" remaining="+length);
        /****************
        * time stamp(s) *
        ****************/
        DataBlock time = new DataBlock("TimeStamp");
        int time_size = time.size()/8;
        if(length>time_size) {
            in.mark(time_size);
            time.read(in);
            int time_code = time.get("type").value().intValue();
            if(time_code==0) {
                /***********************************************
                * no time, just a one byte "no time" indicator *
                ***********************************************/
                in.reset();
                UnsignedIntField no = new UnsignedIntField("no_time",8,
                                              "No time stamps (should be 15)");
                no.read(in);
                add(no);
                length -= 1;
            } else {
                /**************************************************
                * keep this time stamp and see if there's another *
                **************************************************/
                add(time);
                length-= time_size;

                if(time_code==3) {
                    /**************************
                    * read another time stamp *
                    **************************/
                    time = new DataBlock("TimeStamp");
                    time.read(in);
                    add(time);
                    length -= time_size;
                }
            }
        }


        /**************
        * packet data *
        **************/
        if(length>0) {

            RawDataBlock raw = new RawDataBlock(length);
            raw.read(in);
            add(raw);
        }
    
    } else {
        /***************
        * unknown type *
        ***************/
        description="Unknown";
        RawDataBlock raw = upToNextStartCode(in);
        add(raw);
    }

        

       


} // end of read method
        

/****************************************************************************
* read up to the next start code
****************************************************************************/
private RawDataBlock upToNextStartCode(BitInputStream in) throws IOException {

    /**********************************************************
    * unknown type read binary data up to the next start code *
    **********************************************************/
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    for(int good_bytes=0; good_bytes<3; ) {
        
        if(good_bytes==0) in.mark(3);
        byte b = in.readByte();
        out.write(b);
        if( (good_bytes==0 && b == 0) ||
            (good_bytes==1 && b == 0) ||
            (good_bytes==2 && b == 1)   ) {
            /******************
            * this looks good *
            ******************/
            ++good_bytes;
        } else {
            /***********
            * no match *
            ***********/
            good_bytes=0;
        }
    }

    /***************************
    * push back the start code *
    ***************************/
    in.reset();

    byte[] array = out.toByteArray();
    RawDataBlock raw = new RawDataBlock(array);
    raw.resize(array.length-3);
    return raw;
} // end of 


    
/****************************************************************************
*
****************************************************************************/
public String toString() { 

    DataBlock start = (DataBlock)get(0);
    return "MPEG Packet id="+ start.get("type").formattedValue()+
           " "+description; 

}

/****************************************************************************
* inner class for holding an odd number of bits between the end of a heder
* and the next start code.
****************************************************************************/
public class Remainder() extends DataBlock {

int extra_bits;

UnsignedIntField align;
RawDataBlock whole;

public void read(in) {

    extra_bits = in.getBitsToAlign();
    align = new UnsignedIntField("align", extra_bits,
                                 extra_bits+" padding bits");
    align.read(in);
    add(align);

    whole = upToNextStartCode(in);
    add(raw);


} // end of read method

public BitInputStream bitStream() {

    byte[] odd = new byte[1];
    odd[0] = align.value().byteValue();

    BitInputStream in = new BitInputStream(
                        new SequenceInputStream(
                        new ByteArrayInputStream(odd),
                        new ByteArrayInputStream(raw.getByteArray()) ));

    in.readBits(8-extra_bits);

    return in;

}


public String toString() { return "Unparsed Data"; }

} // end of Remainder embedded class

} // end of MPEGPacket class
