import java.io.*;

/***************************************************************************
* This is a class for reading an MPEG Systems stream and extracting
* the embedded streams.
* At least for now it tosses away time code and synchronization information
* It is actually the output end of a pipe. The input end is filled
* by reading from a BitInputStream in separate thread.
**************************************************************************/
public class SystemExtractInputStream extends PipedInputStream 
                                      implements Runnable {

BitInputStream in;
int id;


/****************************************************************************
* Create a new object and start the extracting thread
****************************************************************************/
public SystemExtractInputStream(BitInputStream in, int id) {

    this.in = in;
    this.id = id;

    /****************************
    * start the decoding thread *
    ****************************/
    new Thread(this).start();
    waitUntilPipeReady();


} // end of constructor


/****************************************************************************
*
****************************************************************************/
synchronized private  void signalPipeReady() { notifyAll(); }

synchronized private  void waitUntilPipeReady() {

    try { wait(); } 
    catch(InterruptedException e) {}
}

/****************************************************************************
*
****************************************************************************/
public void run() {

PipedOutputStream out=null;

try {

    /****************************************
    * create a pipe connected to this class *
    ****************************************/
    out = new PipedOutputStream(this);
    signalPipeReady();

    /****************************************
    * read packets until we hit end of file *
    ****************************************/
    while(true) {
        MPEGPacket p = new MPEGPacket();
        p.read(in);
        
        int code = p.getStartCode();
        if(code == id) {
            /*******************************************************
            * this is one of the stream packets we are looking for *
            *******************************************************/
            RawDataBlock data = (RawDataBlock)p.get(p.getChildCount()-1);
            out.write(data.getByteArray());
        }
    }

} catch(EOFException e) {
    /*****************************************************
    * close the pipe on an EOF
    * if the close fails, just kill the thread
    * which will probably throw an IOException on the 
    * reading side
    *****************************************************/
    if(out==null) return;

    try { out.close(); }
    catch(IOException e2) { return; }

} catch(IOException e) {
    /**************************************************************
    * on error, flush the output stream and return to kill the
    * thread. That will cause an IOException to be thrown
    * in the reader's thread.
    *************************************************************/
    if(out==null) return;

    try { out.flush(); }
    catch(IOException e2) {}
    return;
}



} // end of run method


} // end of SystemExtractInputStream class
