import java.io.*;

public class WordStream extends DataStream {

DataBlock header;

/****************************************************************************
* read a single "unit" of data from the stream
****************************************************************************/
protected DataNode readUnit(BitInputStream in) throws IOException {

//TextBlock block = new TextBlock(512);
RawDataBlock block = new RawDataBlock(OLEStream.BIG_BLOCK_SIZE);
block.read(in);
return block;

} // end of readUnit method 

/****************************************************************************
*
****************************************************************************/
public void readHeader(BitInputStream in) throws IOException {

    /**********************************
    * read the File Information Block *
    **********************************/
    header = new DataBlock("FIB");
    header.read(in);
    add(header);

    /************************************************
    * read padding data up to the start of the text *
    ************************************************/
    int text_start_offset = header.get("fcMin").value().intValue();
    
    int skip = text_start_offset - size()/8;
    RawDataBlock padding = new RawDataBlock(skip);
    padding.read(in);
    add(padding);


    /****************************
    * add all the sub-documents *
    ****************************/
    addSubDoc(in, "ccpText", "Main Text");
    addSubDoc(in, "ccpFtn" , "Footnotes");
    addSubDoc(in, "ccpHdd" , "Header Text");
    addSubDoc(in, "ccpAtn" , "Annotation");
    addSubDoc(in, "ccpEdn" , "End Notes");
    addSubDoc(in, "ccpTxbx", "Text Box");
    addSubDoc(in, "ccpHdrTxbx", "Header Text Box");

    /******************************************
    * padding up to the end of the next block *
    ******************************************/
    padToEndOfBlock(in);
    



System.out.println("parsed bytes="+size()/8);




} // end of readHeader method

/****************************************************************************
*
****************************************************************************/
private void addSubDoc(BitInputStream in, String mnemonic, String name) 
             throws IOException {

int length = header.get(mnemonic).value().intValue();

if(length>0) {
    SubDoc doc = new SubDoc(length,name);
    doc.read(in);
    add(doc);
}

} // end of addSubDoc method

/****************************************************************************
*
****************************************************************************/
private void padToEndOfBlock(BitInputStream in) throws IOException {

int block = OLEStream.BIG_BLOCK_SIZE;

int length = size()/8;
int nblocks = (length+block-1)/block;

int pad_size = nblocks * block - length;

RawDataBlock padding = new RawDataBlock(pad_size);
padding.read(in);
add(padding);

} // end of padToEndOfBlock method

} // end of WordStream class
