import java.io.*;
import java.util.*;

/*****************************************************************************
* This class extends OnePassOLEStream to read some additional information
* from the file needed for extracting streams
*****************************************************************************/
public class OLEFile extends OnePassOLEStream {

public static final int SMALL_BLOCK_SIZE=64;
public static final int MAX_SMALL_BLOCK_STREAM_SIZE=4095;

protected StreamDirectory directory;
protected SmallBlockDepot small_depot;
protected DataBlock small_blocks;

/*****************************************************************************
*
*****************************************************************************/
public void read(BitInputStream in) throws IOException {

/***************************
* read the one pass stream *
***************************/
super.read(in);

/****************************
* read the stream directory *
****************************/
BitInputStream in_root = new BitInputStream(getRootStream() );
directory = new StreamDirectory();
directory.read(in_root);
in_root.close();

/*****************************
* read the small block depot *
*****************************/
int first_block = head.getSmallBlockDepotStartBlock();
BitInputStream in_sbd = new BitInputStream(getStream(first_block) );
small_depot = new SmallBlockDepot(blocksInStream(first_block));
small_depot.read(in_sbd);


/********************************
* now read the small block data *
********************************/
first_block = directory.getSmallBlockDataStart();
BitInputStream in_data = new BitInputStream(getStream(first_block) );

small_blocks = new DataBlock();
try {
    while(true) {
        RawDataBlock raw = new RawDataBlock(SMALL_BLOCK_SIZE);
        raw.read(in_data);
        small_blocks.add(raw);
    }
} catch(EOFException e) {} // this is normal


} // end of read method

/****************************************************************************
* return a stream of small block data starting at the given small block
****************************************************************************/
InputStream getSmallBlockStream(int first_block) {

return new SequenceInputStream(new SmallBlockReaderEnumeration(first_block) );

} // end of getSmallBlockStream method

/****************************************************************************
* return the data stream corresponding to the given property
****************************************************************************/
public InputStream getStream(Property prop) {

    int type        = prop.get("type"       ).value().intValue();
    int first_block = prop.get("start_block").value().intValue();
    int size        = prop.get("size"       ).value().intValue();

    if(size==0) return null;

    if(type==5 || size>MAX_SMALL_BLOCK_STREAM_SIZE) {
        return getStream(first_block);
    } else {
        return getSmallBlockStream(first_block);
    }

} // end of getStream (for property) method

/****************************************************************************
* 
****************************************************************************/
public StreamDirectory getStreamDirectory() { return directory; }


/****************************************************************************
*****************************************************************************
* This is an inner class used for linking together the data from
* a series of small blocks into a single data stream
****************************************************************************/
public class SmallBlockReaderEnumeration implements Enumeration {

int index;

SmallBlockReaderEnumeration(int first_block) { index = first_block; }


public boolean hasMoreElements() { return index >= 0; }

public Object nextElement() {

//System.out.println("about to read from small block index "+index);

    RawDataBlock raw = (RawDataBlock)small_blocks.get(index);
    InputStream in = new ByteArrayInputStream(raw.getByteArray());
    index = small_depot.nextBlock(index);

    return in;
} // end of nextElement method

} // end of BlockReaderEnumeration inner class


} // end of OLEFile stream
