import java.io.*;
import java.util.*;

public class OnePassOLEStream extends DataStream {

public static final int BIG_BLOCK_SIZE=512;

protected Block0 head;
int index;

protected BigBlockDepot depot;

/******************************************************************************
* constructor
******************************************************************************/
public OnePassOLEStream() {

super(new RawDataBlock(BIG_BLOCK_SIZE, true));


} // end of constructor

/****************************************************************************
* read a single "unit" of data from the stream
****************************************************************************/
protected DataNode readUnit(BitInputStream in) throws IOException {

DataNode block;

int bbd_index = head.bigBlockDepotIndex(index);
if(bbd_index>=0 ) {

    block = new BigBlockDepotBlock(bbd_index);
    block.read(in);
    depot.add((BigBlockDepotBlock)block, bbd_index);
} else {

    block = new RawDataBlock(BIG_BLOCK_SIZE);
    block.read(in);
}

    ++index;
    return block;


} // end of readUnit method


/****************************************************************************
* read a single "unit" of data from the stream
****************************************************************************/
protected void readHeader(BitInputStream in) throws IOException {

head = new Block0();
head.read(in);
add(head);



depot = new BigBlockDepot(head.getBigBlockDepotCount());

index=0;

} // end of readHeader method

/****************************************************************************
* 
****************************************************************************/
public RawDataBlock getBlock(int index) {

    return (RawDataBlock)get(index+1);

} // end of getBlock method



/****************************************************************************
* returns a data stream starting at a given data block
****************************************************************************/
public InputStream getStream(int first_block) {

    return new BufferedInputStream(
           new SequenceInputStream(
           new BlockReaderEnumeration(first_block) ));
}

/****************************************************************************
* returns the root stream
****************************************************************************/
public InputStream getRootStream() { 

    return getStream(head.getRootStartBlock()); 
} 


/****************************************************************************
* return the number of blocks in a stream starting at the given first block
****************************************************************************/
public int blocksInStream(int first_block) {


int count=0;

for(int index = first_block; index>=0; index = depot.nextBlock(index) ) {
    ++count;

}

return count;
    

} // end of blocksInStream method

/****************************************************************************
*****************************************************************************
* This is an inner class used for linking together the data from
* a series of blocks into a single data stream
****************************************************************************/
public class BlockReaderEnumeration implements Enumeration {

int index;

BlockReaderEnumeration(int first_block) { index = first_block; }


public boolean hasMoreElements() { return index >= 0; }

public Object nextElement() {

//System.out.println("about to read from big block "+index);
    InputStream in = new ByteArrayInputStream(getBlock(index).getByteArray());
    index = depot.nextBlock(index);

    return in;


}

} // end of BlockReaderEnumeration inner class


} // end of OnePassOLEStream method



