import java.io.*;
import java.util.*;

/*****************************************************************************
* This stream splits out all of the data streams in an OLE file
* and presents them sequentially
*****************************************************************************/
public class SplitOLEStream extends DataStream {

OLEFile file;


/****************************************************************************
* recursive funtion to add this property to the tree and all of
* its directory contents
****************************************************************************/
private void addProperty(DataBlock parent, Property property) 
             throws IOException{

    StreamDirectory directory = file.getStreamDirectory();

    String type = property.getType();
    String name = property.get("name").formattedValue();

    DataBlock container = new DataBlock();
    container.name(type+": "+name);
    container.add(property);

    if(property.isDirectory() ) {
        /******************************
        * add the contents of this directory *
        *************************************/
        List list = directory.listDirectory(property);
        for(Iterator it=list.iterator(); it.hasNext(); ){
            Property child = (Property)it.next();
            addProperty(container, child);
        }
    } else {
        /*********************************
        * add the contents of the stream *
        *********************************/
        InputStream byte_stream = file.getStream(property);
        if(byte_stream != null) {
            BitInputStream in2 = new BitInputStream(byte_stream);
            DataStream data = whichDataStream(property);
            data.read(in2);

            container.add(data);
        }
    }

    /***********************************************
    * add the container we just made to the parent *
    ***********************************************/
    parent.add(container);

} // end of addProperty method



/****************************************************************************
* 
****************************************************************************/
public void read(BitInputStream in) throws IOException {

/******************************
* read the OLE file structure *
******************************/
file = new OLEFile();
file.read(in);

StreamDirectory directory = file.getStreamDirectory();

addProperty(this, directory.getRootProperty() );

/*******************************************************
* loop over all the streams, excluding the root stream *
*******************************************************/
for(int i=1; i<0; ++i) {
//for(int i=1; i<directory.getPropertyCount(); ++i) {


    DataBlock stream = new DataBlock();
    Property property = directory.getProperty(i);
    String name = property.get("name").formattedValue();

    stream.name("Stream "+i+" "+name);

    /******************************************************
    * add the property to the stream, but we have to make
    * a copy of it first, otherwise we remove it from the 
    * directory when we put it into our tree
    ******************************************************/
    stream.add(property);

    /*************************************
    * now read the data if there are any *
    *************************************/
    InputStream byte_stream = file.getStream(property);
    if(byte_stream != null) {
        BitInputStream in2 = new BitInputStream(byte_stream);
        DataStream data = whichDataStream(property);
        data.read(in2);

        stream.add(data);
    }

    add(stream);

} // end of loop over streams

} // end of read method



/*****************************************************************************
* return the stream which should be used to read a given property
* subclass can override this method if they knw a better way.
*****************************************************************************/
protected DataStream whichDataStream(Property property) throws IOException {

System.out.println("generic whichDataStream");

    return new BinaryDataStream();

}

} // end of SplitOLEStream
