import java.io.*;
import java.util.*;

public class StreamDirectory {

List properties;

/****************************************************************************
*
****************************************************************************/
public StreamDirectory() { 

    properties = new ArrayList();

} // end of constructor

/****************************************************************************
* read the properties
****************************************************************************/
public void read(BitInputStream in) throws IOException {

properties.clear();

try {
    while(true) {
        Property property = new Property();
        property.read(in);
        if(property.get("name_length").value().intValue() >0 ) {
            /*******************************************
            * this is a valid property and not padding *
            *******************************************/
            properties.add(property);
        }
    }

} catch(EOFException e) {} // this is normal

} // end of read method


/****************************************************************************
* returns the number of properties
****************************************************************************/
public int getPropertyCount() { return properties.size(); }


/****************************************************************************
* return the the root property
****************************************************************************/
public Property getRootProperty() {

    for(int i=0; i<getPropertyCount(); ++i) {

        Property prop = getProperty(i);
        if(prop.isRoot()) return prop;
    }

    return null;

} // end of getRootProperty method 

/****************************************************************************
* return the property at a given index
****************************************************************************/
public Property getProperty(int index) {

    return (Property)properties.get(index);

} // end of getProperty method 


/****************************************************************************
* return the first big block of the small block data stream
****************************************************************************/
public int getSmallBlockDataStart() {

    Property prop = getRootProperty();
    return prop.get("start_block").value().intValue();

} // end of getSmallBlockDataStart method


/****************************************************************************
* return a list of the properties in a given directory.
* If the given property is not a directory, returns an empty list
****************************************************************************/
public List listDirectory(Property property) {

List list = new ArrayList();

int child = property.get("directory").value().intValue();
if(child>=0) listAfter(list, getProperty(child));

return list;


} // end of listDirectory method

/****************************************************************************
* Fill a List with all the properties in a given directory which should 
* appear at or after the given property.
* The List should be cleared before calling this fuction.
* This function is recursive, since the properties are listed as a 
* binary tree.
****************************************************************************/
public void listAfter(List list, Property property) {

list.add(property);

int left  = property.get("previous").value().intValue();
int right = property.get("next").value().intValue();

if( left>=0) listAfter(list, getProperty(left ) );
if(right>=0) listAfter(list, getProperty(right) );

} // end of listDirectoryContents method

    
/*****************************************************************************
* returns the parent directory (a.k.a. storage) for a given property.
* Returns null if there is no parent. This is normal for the root property.
*****************************************************************************/
public Property parentDirectory(Property property) {

/*****************************************************************
* examine all the properties to find the one which is the parent *
*****************************************************************/
for(int i=0; i<getPropertyCount(); ++i) {
    
    Property parent = getProperty(i);
    if(parent.isDirectory() && listDirectory(parent).contains(property) ) {
        /***********
        * found it *
        ***********/
        return parent;
    }
}

/*****************
* didn't find it *
*****************/
return null;

} // end of parentDirectory method


} // end of StreamDirectory class
