import java.io.*;
import java.text.*;
import java.awt.*;

class SwiftTime extends DataBlock implements DataField {

private double scale;

/*******
* data *
*******/

String name;
String comment;

DecimalFormat format;

/*****************************************************************************
* constructor
*****************************************************************************/
public SwiftTime() throws IOException {

super("SwiftTimeBlock");
scale = 20e-6;

get("subseconds").maxValue(new Integer((int)(1./scale+.5) -1 ) );

format = new DecimalFormat();
format.setMaximumFractionDigits(6);

} // end of constructor

/*****************************************************************************
* accessors to implement the DataField interface
*****************************************************************************/
public String name(           ) {      return name; }
public void   name(String name) { this.name = name; }

public String comment(              ) {         return comment; }
public void   comment(String comment) { this.comment = comment; }

/*****************************************************************************
* get value
*****************************************************************************/
public Number value() {

try {
    return new Double (get("seconds"   ).value().intValue() +
                       get("subseconds").value().intValue() * scale );
} catch(NullPointerException e) { 
    return null; 
}

} // end of value get method

/*****************************************************************************
* clips off stray decimal places
*****************************************************************************/
public String formattedValue() { 

Number value = value();
if(value==null) return null;

return format.format(value); 

}

/*****************************************************************************
* set value
*****************************************************************************/
public void   value(Number value) {

double time = value.doubleValue();
int    seconds = (int)time;
int subseconds = (int)((time - (double)seconds) /scale +.5);

get(   "seconds").value(new Integer(   seconds));
get("subseconds").value(new Integer(subseconds));



}// end of value set method


/**************************************************************************
* maximum and minimum value accessors
**************************************************************************/
public Number minValue() {
    return new Double (get("seconds"   ).minValue().intValue() +
                       get("subseconds").minValue().intValue() * scale );
}

public Number maxValue() {
    return new Double (get("seconds"   ).maxValue().intValue() +
                       get("subseconds").maxValue().intValue() * scale );
}



public void minValue(Number min) {}
public void maxValue(Number max) {}




/*****************************************************************************
* string representation
*****************************************************************************/
public String toString() {

return  name+" = "
       +formattedValue() +" / "
       +comment; 

} 

/**********************************************************************
* component for rendering 
**********************************************************************/
public Component displayComponent(boolean selected, boolean expanded, 
                                  int row, boolean hasFocus) {

    return new DataFieldRenderer(this);

} // end of displayComponent method



/***************************************************************************
* editing methods 
***************************************************************************/
public DataNodeEditor getEditor(int row) {

    return new DataFieldRenderer(this,true);

}

/***************************************************************************
* allow a new value to be set  - called by tree after editing
***************************************************************************/
public void setUserObject(Object o) { 

try {
    value(new Double(o.toString() ));
    return;
} catch (NumberFormatException e) {}

} // end of setUserObject method


} // end of SwiftTime class
